#!/usr/bin/env python2.5
# -*- coding: utf-8 -*-
###########################################################################
## File:        main.py
## App name:    SleepAnalyser
## Description: Sleep Analyser records your movement during your sleep. It is able to visualise it on a graph to show
##              how much you move during your sleep. This can help to indicate how you sleep. It also has an alarm
##              function. You can set the alarm and a time window (ex. 30 minutes). The Alarm will then go off
##              sometimes during the time window (as soon as you move more), but latest at the set alarm.
##              Old records can be visualised and you can load records from your friends. SleepAnalyser also has a
##              test function. It will record and visualise much faster. Also it will make a beep when ever your
##              movement goes over the trigger level. Other features are lucid dream and wake up music.
## Copyright:   2010-2011 by George Ruinelli, www.ruinelli.ch, george@ruinelli.ch
## Licence:     GPL
## Target OS:    MAEMO 5 or later on N900
##                Runs also on any OS with python installed, how ever with limited functionality
###########################################################################

from PyQt4.QtCore import *
from PyQt4.QtGui import *
import os, sys

import dbus
from dbus.mainloop.qt import DBusQtMainLoop
from time import strftime

import data #own data
import records #all record data is stored in here


#Get application root path
data.app_path=os.path.dirname(__file__)
if(data.app_path==""): data.app_path="./"
else: data.app_path=data.app_path+"/"

print strftime("%A %x %X") #for log file

#Load version file
try:
    file = open(data.app_path+"version", 'r')
    data.version = file.readline()
    data.version=data.version[:-1]
    data.build = file.readline()
    print "SleepAnalyser Version: "+str(data.version)+ "-"+str(data.build)
except:
    print "Version file not found, please check your installation!"

#check parameters
if(len(sys.argv)>1):  #at least one parameter
    print "Parameters:", sys.argv
    if(sys.argv[1]=="debug"):
        print "Enable Debug mode"
        data.debug=True
    elif(sys.argv[1]=="lang"):
        if(len(sys.argv)>2):
            data.language=sys.argv[2]
            print "Set language to:", data.language
        else:
            print "Wrong parameter list"
            print "Please set 2-letter language code"
            exit()

    else: #show command line interface data
        print "sleepanalyser [options]"
        print "OPTIONS:"
        print "  debug: Shows debug information"
        print "  lang COUNTRYCODE: Use specific language file. (Use 2-letter country code like de, en, fr, nl, ..."
        exit()

import i18n #load language file
_ = i18n.language.ugettext

import kht_report  #Error reporting dialog

#own functions
from functions_general import *
from functions_file import *
from functions_record import *
from functions_alarm import *

#own c functions (for speed improvements)
try:
    from lib import  mylib
    MyLib_Imported=True
except:
    MyLib_Imported=False

#GUI files, generated with QT designer
from Ui_StartWindow import *
from Ui_RecordWindow import *
from Ui_ViewWindow import *
from Ui_ConfigWindow import *
from Ui_HelpWindow import *
from Ui_DateWindow import *
from Ui_SetRecordingWindow import *
from Ui_WakeUpSettingsWindow import *
from Ui_LucidDreamSettingsWindow import *
from Ui_AlarmSettingsWindow import *
from Ui_RecordingTerminationWindow import *
from Ui_AdditionalInfoWindow import *




###########################################################################
## Function:    main
## Description: main function, gets called at start
## Parameters:  none
## Returns:     none
###########################################################################
def main():
    global app
    global StartWindow, RecordWindow, ViewWindow, ConfigWindow, DateWindow, SetRecordingWindow, LucidDreamSettingsWindow, WakeUpSettingsWindow, AlarmSettingsWindow, RecordingTerminationWindow, AdditionalInfoWindow, HelpWindow
    global config

    #if not on the N900, the acceleration sensor has to be simulated
    if(os.path.exists(str(data.AccSensorPath))): data.DeviceIsN900=True    #runs on real N900 hardware
    else: data.DeviceIsN900=False    #runs on other hardware/OS, deactivate features, simulate some functions


    app = QApplication(sys.argv)

    kht_report.install_excepthook("SleepAnalyser", data.version +"-"+ data.build) #register error reporter

    if(data.debug==True): print "Application path:", data.app_path

    if(data.DeviceIsN900==False):
        print "Runs not on a N900 => Use randomizer for acceleration sensor data"
        data.configFolder=data.developmentconfigFolder
        data.RecordFolder=data.developmentRecordFolder
    else: data.RecordFolder=data.defaultRecordFolder

    config=LoadConfig()

    #check if c module is available and can get loaded
    if(MyLib_Imported==True): #module is loaded
        print "Mylib is compiled:", mylib.IsCompiled() #check if module is compiled
        if(mylib.IsCompiled()==False): #module is not compiled
            txt= "The compiled module mylib.so could not get loaded! Please check if there is a never version "+ \
                     "or inform the maintainer about it! SleepAnalyser will still work, " + \
                     "but some functions will work slow as we have to fall back to the unoptimised python code!"
            print txt
            cmd="dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteDialog " + \
                     "string:\""+"SleepAnalyser:\n"+txt+"\" uint32:0 string:\"OK\""
            os.popen(cmd)
    else: #module is available but broken
        txt="SleepAnalyser could not load the module mylib.so\n" + \
                 "in "+data.app_path+"lib. It might be corrupted. Please check if there is a never version "+ \
                 "or inform the maintainer about it! You can get SleepAnalyser working by removing the file "+data.app_path+"lib/mylib.so, " + \
                 "how ever some functions will work slow as we have to fall back to the unoptimised python code!"
        print txt
        cmd="dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteDialog " + \
                 "string:\""+"SleepAnalyser:\n"+txt+"\" uint32:0 string:\"OK\""
        os.popen(cmd)
        exit()

    #Load Stylesheet
    try:
        file = open(data.configFolder+"UIStyle.css_user.css", 'r')
        data.UIStyle=""
        for line in file: data.UIStyle=data.UIStyle+line+"\n"
        file.close()
        print "Loaded user defined style sheet"
    except:
        print "User defined style sheet not existing or not readable, load default one"
        print "If you want to use a user defined style sheet, place it under", data.configFolder+"UIStyle_user.css"
        try:
            file = open(data.app_path+"UIStyle", 'r')
            data.UIStyle=""
            for line in file: data.UIStyle=data.UIStyle+line+"\n"
            file.close()
        except:
            pass
#        try:
#            print "Create user defined style sheet in", data.configFolder
#            file= open(data.configFolder+"UIStyle.css", 'w')
#            file.write(data.UIStyle)
#            file.close()
#        except:
#            print "Could not create user defined style sheet"

    #Create main window
    StartWindow = frmStartWindow()
    StartWindow.show()

    #Create subwindows
    RecordWindow = frmRecordWindow(StartWindow)
    ConfigWindow = frmConfigWindow(StartWindow)
    ViewWindow = frmViewWindow(StartWindow)
    HelpWindow = frmHelpWindow(StartWindow)
    SetRecordingWindow = frmSetRecordingWindow(StartWindow)
    AlarmSettingsWindow = frmAlarmSettingsWindow(SetRecordingWindow)
    LucidDreamSettingsWindow = frmLucidDreamSettingsWindow(SetRecordingWindow)
    WakeUpSettingsWindow = frmWakeUpSettingsWindow(SetRecordingWindow)
    DateWindow = frmDateWindow(ViewWindow)
    AdditionalInfoWindow = frmAdditionalInfoWindow(ViewWindow)
    RecordingTerminationWindow = frmRecordingTerminationWindow(RecordWindow)

    #Hide subwindows at first
    RecordWindow.hide()
    ViewWindow.hide()
    ConfigWindow.hide()
    SetRecordingWindow.hide()
    AlarmSettingsWindow.hide()
    LucidDreamSettingsWindow.hide()
    WakeUpSettingsWindow.hide()
    DateWindow.hide()
    AdditionalInfoWindow.hide()
    RecordingTerminationWindow.hide()

    if(data.user==""):#get user name for in record files
        EditUserName()

    DisplayStatusDaemon()
    GetBrightness() #get current display brightness

    delete_all_Alarms() #Check if there are old alarms. If SleepAnalyser crashed, they might still be set
    
    sys.exit(app.exec_())




###########################################################################
## Function:    EditUserName
## Description: Opens a dialog to edit user name
## Parameters:  none
## Returns:     none
###########################################################################
def EditUserName():
    r = QInputDialog.getText(None, _("SleepAnalyser"), _("Please enter your name.\nIt will be used in the records file, this makes it easier\nwhen you want to share your sleep pattern with others."), 0, data.user)
    if(r[1]==True): #OK pressed
        data.user=unicode(r[0])
        if(data.user==""): data.user="-"
        config.set('main', 'user', str(data.user.encode( "utf-8" )))

    try:
        print "User name:", data.user
    except:
        pass








###########################################################################
## Function:    CheckFileValidity
## Description: checks if file/folder does not contain illegal characters
## Parameters:  file or folder path
## Returns:     none
###########################################################################
def CheckFileValidity(file):
    print "Check if file/folder does not contain % or ,"
    if(string.find(file,  "%")!=-1 or string.find(file,  ",")!=-1):
        print "Illegal folder name!"        
        QtGui.QMessageBox.warning(None, _("SleepAnalyser"), _("Illegal characters in file or folder!"), QtGui.QMessageBox.Ok)
        return 0
    return 1









###########################################################################
## Function:    Play
## Description: Play selected song
## Parameters:  path to file or folder, volume
## Returns:     none
###########################################################################
def Play(file,  volume):
    if(not os.path.exists(str(file))):
        print "Sound file/folder not valid"
        QtGui.QMessageBox.warning(None, _("SleepAnalyser"), _("Please first select a valid song file or folder!"), QtGui.QMessageBox.Ok)
        return
    
    if(os.path.isfile(str(file))): #file
        f=file
    else: #folder
        print file, "is a folder, select random file in it"
        
        arr=getFileList(file,  data.song_file_types) 
        amount=len(arr)
        if(amount==0):
            print "No sound files in this folder"
            QtGui.QMessageBox.warning(None, _("SleepAnalyser"), _("No sound files in this folder.\nPlease first select another folder!"), QtGui.QMessageBox.Ok)
            return
        i=int(random.uniform(0,  amount))
        f=file+"/"+arr[i]
        print i,  f
        
    Notify(_("Open Song in media player, set volume to ")+ str(volume) + "%")
    PlaySong(volume, f)






###########################################################################
## Function:    MeasurementTimerLoop
## Description: This routine gets called from the timer during the recording
##              Interval: sampling_interval => 1000 ms or 1 ms if in quick mode
##              In every run it initiates a sampling series, the proccessing
##              of the data and needed actions.
## Parameters:  none
## Returns:     none
###########################################################################
def MeasurementTimerLoop():
    r=GetSensorData()
    if(r==1): #new data got added to array
        GenerateDerivation(False) #Proccess new recorded data
        GenerateSleepPattern() #Recalculate sleep pattern curve (Note: This function is very time consuming)
        ProcessAlarm() #check if alarm should get initiated

        WriteRecord() #Write recorded data into record data file

        data.ScrollToEnd=True
        if(data.DisplayIsOn==True): #when display is on, update graph
            GenerateStatisticData()
            Generate_and_show_GraphBitmap(RecordWindow)

        if(records.Index==records.MaxArrayDataLen-1):
            print "Memory is full!"
            QtGui.QMessageBox.warning(RecordWindow, _("SleepAnalyser"), _("You reached the end of the recording capacity!\nThe recording stops now."), QtGui.QMessageBox.Ok)
            StopMeasurement() #stop but keep alarm active!
            SwitchModes(False) #change back to normal mode and general profile if requested

    ProcessAlarm() #check if alarm should get initiated
    ProcessLucid_Dream() #check if lucid dream should get initiated

    if(records.Index==1): 
        SwitchModes(True) #change to offline mode and silent profile if requested
        #above commands are delayed from the start to here, so we get a smoother visual change of windows


    if(data.DisplayIsOn==True):#only when display is on, update clock animation
        if(data.debug==True): print "-> o Update screen"
        RecordWindow.UpdateAnimation() #update turning clock
        RecordWindow.UpdateScreen()


    if(data.AlarmIsActive==True): #alarm got activated
        if(data.StopAfterAlarm==True): #stop recording immediately
            data.DisplayIsOn=True #Workaround, so display gets updated even display is off

            print "Alarm got activated, stopped recording"
            txt=_("Alarm got activated, stopped recording")
            data.Status=txt

            StopMeasurement() #stop but keep alarm active!

            #change directly to view screen
            data.LastRecord=data.RecordFilename #set to corrent file for viewwindow
            config.set('main', 'LastRecord', str(data.LastRecord))
            print "Show record in ViewWindow:", data.LastRecord
            RecordWindow.hide()
            StartWindow.ShowViewWindow()

        else: #continue recording
            pass

        SwitchModes(False) #change back to normal mode and general profile if requested




###########################################################################
## Function:    InitMeasurement
## Description: It initiates the measurement and sets all needed parameters
## Parameters:  none
## Returns:     none
###########################################################################
def InitMeasurement():
    global app

    data.sample_index=0
    data.last_beep=0 #used for quick mode

    data.WakeUp_SongIsActive=False
    data.Lucid_Dream_SongIsActive=False

    data.Animation=7 #start position fur turning clock pointer
    RecordWindow.UpdateAnimation() #show turning clock

    if(data.QuickMode==False): data.Status=_("Recording...")
    else: data.Status=_("Recording in Test Mode...")

    records.User=data.user
    records.SWversion=data.version
    records.Sampling_interval=data.Sampling_interval
    records.Sample_smoothing=data.Sample_smoothing
    records.Value_smoothing=data.Value_smoothing
    records.Trigger=data.Trigger #save Trigger level on start
    records.statusID="OK"

    if(data.debug==True): PrintRecordData()

    #set graph X scale and sampling interval
    if(data.QuickMode==False):
        data.interval=data.Sampling_interval #normal sampling rate
        data.Xscale=1 #set to default zoom level
    else:
        data.interval=10 #Quick mode for testing
        data.Xscale=0 #set to max zoom level

#    print "#################### USE MAXIMUM SAMPLING RATE FOR DEBUGGING!!!!!!!!!!!"
#    Note: Even it is told to run with 1 ms, in reality it will be much slower!
#    data.interval=1 #Debugging

    data.run=True #recording
    app.timer_GetData = QtCore.QTimer()
    QtCore.QObject.connect(app.timer_GetData, QtCore.SIGNAL("timeout()"), MeasurementTimerLoop)

    data.StartTime=time.localtime() #store recording start time
    data.StartTimeSec=Tuple2Seconds(data.StartTime)

    app.timer_GetData.start(data.interval) #start timer

    CreateRecordFile() #Create record data file
    GetFirstEntry() #generate a dummy first entry for record data file
    WriteRecord() #Write recorded data into record data file

    Generate_and_show_GraphBitmap(RecordWindow)

    if(records.AlarmIsSet==True or data.QuickMode==True): #only show the trigger lower/raise buttons when alarm is set or in quick mode
        RecordWindow.ui.bRaiseTrigger.show()
        RecordWindow.ui.bLowerTrigger.show()
    else:
        RecordWindow.ui.bRaiseTrigger.hide()
        RecordWindow.ui.bLowerTrigger.hide()

    RecordWindow.ui.lblStatus.hide()
    RecordWindow.ui.CanvasWidget.show()
    RecordWindow.UpdateScreen()
    app.processEvents() #force ui update

#    SwitchModes(True) #change to offline mode and silent profile if requested




###########################################################################
## Function:    StopMeasurement
## Description: Stops the timer for the measurement, updates data
## Parameters:  none
## Returns:     none
###########################################################################
def StopMeasurement():
    global app

    GenerateStatisticData()
    Generate_and_show_GraphBitmap(RecordWindow)

    RecordWindow.Canvas.update()
    RecordWindow.UpdateScreen()
    app.processEvents() #force ui update

    if(data.run==True): #in recording
        if(data.debug==True): print "Stop Timer"
        app.timer_GetData.stop()
        StopRecordFile()
        #PauseSong() #stop song, in case it was playing
        CloseMediaPlayer() #close media player
        UpdateFileHeader() #rewrite record file (only do when recording is stopped!)

    data.run=False

    RecordWindow.UpdateScreenStop()







###########################################################################
## Function: Zoom
## Description: Changes the zoom level in X axis
## Parameters:  window, (direction: +1 (more details) or -1 (less details)), status text
## Returns:     none
###########################################################################
def Zoom(self, direction, text):
    self.ui.lblStatus.setText(text)
    self.ui.lblStatus.show()
    app.processEvents() #force ui update

    if(direction==+1): #zoom in
        if(data.Xscale==0): pass #min zoom level
        else: data.Xscale=(data.Xscale-1) % data.arrXscaleLen
    else: # -1, zoom out
        if(data.Xscale==data.arrXscaleLen-1): pass #max zoom level
        else: data.Xscale=(data.Xscale+1) % data.arrXscaleLen

    Generate_and_show_GraphBitmap(self)
    self.ui.lblStatus.hide()

    if(data.Xscale==0): self.ui.bZoomIn.setEnabled(False)
    else: self.ui.bZoomIn.setEnabled(True)
    if(data.Xscale==data.arrXscaleLen-1): self.ui.bZoomOut.setEnabled(False)
    else: self.ui.bZoomOut.setEnabled(True)

    app.processEvents() #force ui update




###########################################################################
## Function:    MoveTrigger
## Description: Raise or lower the Trigger level
## Parameters:  window, (direction: +1 or -1), status text
## Returns:     none
###########################################################################
def MoveTrigger(self, direction, text):
    self.ui.lblStatus.setText(text)
    self.ui.lblStatus.show()
    app.processEvents() #force ui update

    if(direction==+1): #up
        data.Trigger=data.Trigger+1
        if(data.Trigger>data.maxTrigger): data.Trigger=data.maxTrigger
    else: #-1, down
        data.Trigger=data.Trigger-1
        if(data.Trigger<data.minTrigger): data.Trigger=data.minTrigger
    if(data.run==True): records.Trigger=data.Trigger

    Generate_and_show_GraphBitmap(self)
    self.ui.lblStatus.hide()

    if(data.Trigger==data.maxTrigger): self.ui.bRaiseTrigger.setEnabled(False)
    else: self.ui.bRaiseTrigger.setEnabled(True)
    if(data.Trigger==data.minTrigger): self.ui.bLowerTrigger.setEnabled(False)
    else: self.ui.bLowerTrigger.setEnabled(True)

    app.processEvents() #force ui update




###########################################################################
## Function:    About
## Description: Shows a dialog box with copyright info
## Parameters:  none
## Returns:     none
###########################################################################
def About():
    QtGui.QMessageBox.about(None, _("About")+" "+_("SleepAnalyser")+" "+str(data.version)+"-"+str(data.build), _("Copyright 2010-2011 by George Ruinelli with great support from the MAEMO community")+"\n"+\
    _("Contact:")+" george@ruinelli.ch\n" +\
    _("Website:")+" http://www.ruinelli.ch/sleepanalyser/\n\n"+\
    _("Translations:")+"\n" +\
    data.translation_copyrights)




###########################################################################
## Function:    Donation
## Description: Shows a dialog box with donation info
## Parameters:  none
## Returns:     none
###########################################################################
def Donation():
    QtGui.QMessageBox.about(None, _("SleepAnalyser"), _("SleepAnalyser is available free of charge. How ever if you like it, I would appreciate a donation. You can donate either via PayPal to george@ruinelli.ch or by using my IBAN")+" (CH92 0070 0113 3000 3973 8).")













###########################################################################
###########################################################################
## Function:    Class for main window
## Description: All functions needed for the main window
##                This window is shown after application start
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmStartWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_StartWindow()
        self.ui.setupUi(self)

        #signals
        QtCore.QObject.connect(self.ui.bStartRecording, QtCore.SIGNAL("clicked()"), self.StartRecording)
        QtCore.QObject.connect(self.ui.bView, QtCore.SIGNAL("clicked()"), self.ShowViewWindow)

        QtCore.QObject.connect(self.ui.actionTest_recording, QtCore.SIGNAL("triggered()"), self.StartMeasurementQuick)
        QtCore.QObject.connect(self.ui.actionSettings, QtCore.SIGNAL("triggered()"), self.ShowConfigWindow)
        QtCore.QObject.connect(self.ui.actionAbout, QtCore.SIGNAL("triggered()"), About)
        QtCore.QObject.connect(self.ui.actionDonation, QtCore.SIGNAL("triggered()"), Donation)
        QtCore.QObject.connect(self.ui.actionHelp, QtCore.SIGNAL("triggered()"), self.ShowHelpWindow)

        self.setStyleSheet(data.UIStyle) #Add styles

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.lblIcon.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"SleepAnalyser2.png"))
            self.ui.bStartRecording.setIcon(QIcon(data.app_path+"img/"+"krec.png"))
            self.ui.bView.setIcon(QIcon(data.app_path+"img/"+"zoom.png"))

        #set texts
        self.setWindowTitle(_("SleepAnalyser"))
        self.ui.bView.setText(_("View records"))
        self.ui.bStartRecording.setText(_("Start recording"))
        self.ui.actionAbout.setText(_("About"))
        self.ui.actionDonation.setText(_("Donation"))
        self.ui.actionSettings.setText(_("Settings"))
        self.ui.actionHelp.setText(_("Help"))
        self.ui.actionTest_recording.setText(_("Test recording"))





    ###########################################################################
    ## Function:    StartMeasurementQuick
    ## Description: Menu event: Starts the meassurement in quick mode
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def StartMeasurementQuick(self): ##(Class StartWindow)
        if(data.debug==True): print "Measure in QuickMode"
        data.QuickMode=True #run in quick mode (for testing of functionality only, no data gets stored)

        RecordWindow.ui.lblStatus.hide()
        RecordWindow.show()
        app.processEvents() #force ui update

        InitRecord() #clear all old data
        InitMeasurement()
        RecordWindow.UpdateScreenStart()
        RecordWindow.Canvas.show()





    ###########################################################################
    ## Function:    ShowViewWindow
    ## Description: Button event: Open view window with latest recorded record
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowViewWindow(self): ##(Class StartWindow)
        ViewWindow.ui.lblStart.setText("")
        ViewWindow.ui.lblEnd.setText("")
        ViewWindow.ui.lblNote.setText("")
        ViewWindow.Canvas.hide()
        ViewWindow.StatisticsCanvas.hide()
        ViewWindow.show()
        ViewWindow.UpdateScreenStart()
        app.processEvents() #force ui update

        #reset to default record folder
        if(data.DeviceIsN900==False): data.RecordFolder=data.developmentRecordFolder #folder on devel PC
        else: data.RecordFolder=data.defaultRecordFolder

        LoadFileList()
        data.arrRecordFilesIndex=len(data.arrRecordFiles)-1 #no file selected, show latest

        if(os.path.exists(str(data.RecordFolder+data.LastRecord))):
            data.RecordFilename=data.LastRecord
        else:
            l=len(data.arrRecordFiles)
            if(l>0):
                data.RecordFilename=data.arrRecordFiles[l-1] #open last (youngest)record file
            else: #no RecordFiles in RecordFolder, show "open file" dialog
                tmp=ViewWindow.ShowFileDialog()
                print "## ShowViewWindow", tmp
                if(not os.path.exists(str(tmp))): #only continue if valid record file selected
                    StartWindow.show()
                    ViewWindow.hide()
                    return #only continue if valid record file selected
                tmp2=tmp.split("/")
                l=len(tmp2)
                data.RecordFilename=tmp2[l-1]
                data.RecordFolder=string.replace(tmp, data.RecordFilename, "") #get folder of selected file
                print "Set folder=", data.RecordFolder, ", file=", data.RecordFilename

        ViewWindow.ShowRecord()

        if(data.RecordFilename==""):
            print "No valid file selected"
            ViewWindow.hide()





    ###########################################################################
    ## Function:    ShowConfigWindow
    ## Description: Menu event: Show the config dialog
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowConfigWindow(self): ##(Class StartWindow)
        data.LastWindow="StartWindow"
        ConfigWindow.UpdateScreen()
        ConfigWindow.show()




    ###########################################################################
    ## Function:    ShowHelpWindow
    ## Description: Menu event: Show help window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowHelpWindow(self): ##(Class StartWindow)
        data.LastWindow="StartWindow"
        HelpWindow.load_data()
        HelpWindow.show()





    ###########################################################################
    ## Function:    StartRecording
    ## Description: Button event: Show the recording parameter window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def StartRecording(self): ##(Class StartWindow)
        SetRecordingWindow.UpdateScreenStart()
        SetRecordingWindow.UpdateScreen()
        SetRecordingWindow.show()





    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: Save config, close program
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event): ##(Class StartWindow)
        global config
        WriteConfig()
        delete_all_Alarms() #Check if there are old alarms. If SleepAnalyser crashed, they might still be set

        if(data.DimmDisplay==True): #restore display brightness
            SetBrightness(data.PreviousBrightness)














###########################################################################
###########################################################################
## Function:    Class for recording
## Description: All functions needed for the recording window
##                This window is shown after during recording
##                It is only called from the start window
## Returns:     none
###########################################################################
###########################################################################
class frmRecordWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_RecordWindow()
        self.ui.setupUi(self)
        app.processEvents() #force ui update

        self.Canvas = GraphCanvas(self.ui.CanvasWidget)

        self.ui.CanvasWidget.setMinimumSize(QtCore.QSize(1000, data.GraphBitMapHeight)) #needed?

        #signals
        QtCore.QObject.connect(self.ui.bStop, QtCore.SIGNAL("clicked()"), self.closeEvent)
        QtCore.QObject.connect(self.ui.bRaiseTrigger, QtCore.SIGNAL("clicked()"), self.RaiseTrigger)
        QtCore.QObject.connect(self.ui.bLowerTrigger, QtCore.SIGNAL("clicked()"), self.LowerTrigger)
        QtCore.QObject.connect(self.ui.bZoomIn, QtCore.SIGNAL("clicked()"), self.ZoomIn)
        QtCore.QObject.connect(self.ui.bZoomOut, QtCore.SIGNAL("clicked()"), self.ZoomOut)
        QtCore.QObject.connect(self.ui.bEditNote, QtCore.SIGNAL("clicked()"), self.EditNote)

        QtCore.QObject.connect(self.ui.actionHelp, QtCore.SIGNAL("triggered()"), self.ShowHelpWindow)
        QtCore.QObject.connect(self.ui.actionAbout, QtCore.SIGNAL("triggered()"), About)

        self.setStyleSheet(data.UIStyle) #Add styles

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.bZoomIn.setIcon(QIcon(data.app_path+"img/"+"zoom-in.png"))
            self.ui.bZoomOut.setIcon(QIcon(data.app_path+"img/"+"zoom-out.png"))
            self.ui.bRaiseTrigger.setIcon(QIcon(data.app_path+"img/"+"go-up.png"))
            self.ui.bLowerTrigger.setIcon(QIcon(data.app_path+"img/"+"go-down.png"))
            self.ui.bEditNote.setIcon(QIcon(data.app_path+"img/"+"kwrite.png"))
            self.ui.lblAnimation.setPixmap(QtGui.QPixmap(data.app_path+"img/anim/0.png"))

        #texts
        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Recording")+")")
        self.ui.lblElapsedTime.setText(_("Elapsed time:")+" 0")
        self.ui.bStop.setText(_("Stop"))
        self.ui.lblAlarmTime.setText(_("Alarm time:")+ " "+_("(no alarm set)"))
        self.ui.lblStatus2.setText(_("Status:"))
        self.ui.lblStatus.setText(_("Loading record..."))
        self.ui.lblstate1.setText(_("Awake"))
        self.ui.lblstate2.setText(_("Light sleep"))
        self.ui.lblstate3.setText(_("Deep sleep"))
        self.ui.actionHelp.setText(_("Help"))
        self.ui.actionAbout.setText(_("About"))





    ###########################################################################
    ## Function:    UpdateScreenStart
    ## Description: Update window data on recording start
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreenStart(self): ##(Class RecordWindow)
        t=strftime("%a. %x %X", data.StartTime)
        self.ui.lblElapsedTime.setText(_("Elapsed time:"))
        self.ui.bStop.setText(_("Stop"))
        self.ui.lblNote.setText(_("Note:"))

        if(records.AlarmIsSet==True): #alarm active
            self.ui.lblAlarmTime.setText(_("Alarm time:")+" "+strftime("%H:%M", records.AlarmTimeWindowStart_tuple)+" - "+strftime("%H:%M", records.AlarmTimeWindowEnd_tuple))
        else: #only recording, no alarm
            self.ui.lblAlarmTime.setText(_("Alarm time:")+ " "+_("(no alarm set)"))

        self.ui.bStop.setIcon(QIcon(data.app_path+"img/"+"krec2.png"))

        if(records.Use_Lucid_Dream_Song==True): self.ui.lblIcon1.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"music_green.png"))
        else: self.ui.lblIcon1.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"no_music_green.png"))
        if(records.Use_WakeUp_Song==True): self.ui.lblIcon2.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"music_red.png"))
        else: self.ui.lblIcon2.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"no_music_red.png"))
        if(records.AlarmIsSet==True):
            if(data.UseAlternativeAlarm==False):
                self.ui.lblIcon3.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"SleepAnalyser.png"))
            else:
                self.ui.lblIcon3.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"Alternative_Alarm.png"))
        else: self.ui.lblIcon3.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"Alarm_off.png"))


        if(data.Xscale==0): self.ui.bZoomIn.setEnabled(False)
        else: self.ui.bZoomIn.setEnabled(True)
        if(data.Xscale==data.arrXscaleLen-1): self.ui.bZoomOut.setEnabled(False)
        else: self.ui.bZoomOut.setEnabled(True)

        if(data.Trigger==data.maxTrigger): self.ui.bRaiseTrigger.setEnabled(False)
        else: self.ui.bRaiseTrigger.setEnabled(True)
        if(data.Trigger==data.minTrigger): self.ui.bLowerTrigger.setEnabled(False)
        else: self.ui.bLowerTrigger.setEnabled(True)

        self.UpdateScreen()




    ###########################################################################
    ## Function:    UpdateScreen
    ## Description: Update window data during recording
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreen(self): ##(Class RecordWindow)
        if(data.run==True): #recording
            t=time.mktime(time.localtime())
            time_difference=t-data.StartTimeSec
            d=time.strftime("%Hh %Mm", time.gmtime(time_difference))
            self.ui.lblElapsedTime.setText(_("Elapsed time:")+" "+d)
        else: self.UpdateScreenStop()

        self.ui.lblStatus2.setText(data.Status)





    ###########################################################################
    ## Function:    UpdateScreenStop
    ## Description: Update window data on recording stop
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreenStop(self): ##(Class RecordWindow)
        self.ui.bStop.setText(_("Close"))
        self.ui.lblStatus2.setText(data.Status)





    ###########################################################################
    ## Function:    UpdateAnimation
    ## Description: Update the clock pointer animation during recording
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateAnimation(self):
        data.Animation=(data.Animation+1) % 8
        self.ui.lblAnimation.setPixmap(QtGui.QPixmap(data.app_path+"img/anim/"+str(data.Animation)+".png"))





    ###########################################################################
    ## Function:    ZoomIn
    ## Description: Button event: Zoom in
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ZoomIn(self): ##(Class RecordWindow)
        Zoom(self, +1, _("Zoom in..."))




    ###########################################################################
    ## Function:    ZoomIn
    ## Description: Button event: Zoom in
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ZoomOut(self): ##(Class RecordWindow)
        Zoom(self, -1, _("Zoom out..."))




    ###########################################################################
    ## Function:    RaiseTrigger
    ## Description: Button event: Raise trigger level
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def RaiseTrigger(self): ##(Class RecordWindow)
        MoveTrigger(self, +1, _("Raise trigger level..."))




    ###########################################################################
    ## Function:    LowerTrigger
    ## Description: Button event: lower trigger level
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def LowerTrigger(self): ##(Class RecordWindow)
        MoveTrigger(self, -1, _("Lower trigger level..."))




    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: show terminate window if normal recording,
    ##              if in quick mode, terminate recording directly and delete record
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event=0): ##(Class RecordWindow)
        global RecordingTerminationWindow, StartWindow
        if(data.QuickMode==False ): #running in normal mode
            try:
                event.ignore()  #ignore closing
            except:
                pass
            RecordingTerminationWindow.show()
        else: #quick mode, delete file anyway and close
            StopMeasurement()
            print "Delete RecordFile: "+data.RecordFilename
            try:
                os.remove(data.RecordFolder+data.RecordFilename)
            except:
                print "Can not delete it. It might not exist"
            data.QuickMode=False
            StartWindow.show()
            RecordWindow.hide()
            self.hide()
            SwitchModes(False) #change back to normal mode and general profile if requested






    ###########################################################################
    ## Function:    EditNote
    ## Description: Button event: Opens a dialog to edit note
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def EditNote(self): ##(Class RecordWindow)
        r=QInputDialog.getText(None, _("SleepAnalyser"), _("Add a note to this record\n(leave empty for no note):"), QLineEdit.Normal, records.Note)
        if(r[1]==True): #OK pressed
            records.Note=unicode(r[0])
            self.ui.lblNote.setText(_("Note:")+" "+records.Note)
            if(data.run==True): UpdateFileHeader() #rewrite record file





    ###########################################################################
    ## Function:    ShowHelpWindow
    ## Description: Menu event: Show help window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowHelpWindow(self): ##(Class RecordWindow)
        data.LastWindow="RecordWindow"
        HelpWindow.load_data()
        HelpWindow.show()

















###########################################################################
###########################################################################
## Function:    Class for view window
## Description: All functions needed for the main window
##                This window is shows a record from a file
##                It is only called from the start window, or after a record gets stopped
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmViewWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent=None):
        global app
        QMainWindow.__init__(self, parent)
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_ViewWindow()
        self.ui.setupUi(self)

        self.Canvas = GraphCanvas(self.ui.CanvasWidget)
        self.StatisticsCanvas = StatCanvas(self)

        self.ui.CanvasWidget.setMinimumSize(QtCore.QSize(1000, data.GraphBitMapHeight)) #needed?

        #signals
        QtCore.QObject.connect(self.ui.bZoomIn, QtCore.SIGNAL("clicked()"), self.ZoomIn)
        QtCore.QObject.connect(self.ui.bZoomOut, QtCore.SIGNAL("clicked()"), self.ZoomOut)
        QtCore.QObject.connect(self.ui.bNext, QtCore.SIGNAL("clicked()"), self.NextRecord)
        QtCore.QObject.connect(self.ui.bLast, QtCore.SIGNAL("clicked()"), self.LastRecord)
        QtCore.QObject.connect(self.ui.bDelete, QtCore.SIGNAL("clicked()"), self.DeleteRecord)
        QtCore.QObject.connect(self.ui.bOpen, QtCore.SIGNAL("clicked()"), self.OpenRecord)
        QtCore.QObject.connect(self.ui.bEditNote, QtCore.SIGNAL("clicked()"), self.EditNote)
        QtCore.QObject.connect(self.ui.bOpenDate, QtCore.SIGNAL("clicked()"), self.OpenDate)
        QtCore.QObject.connect(self.ui.bStatistics, QtCore.SIGNAL("clicked()"), self.ShowRecordData)

        QtCore.QObject.connect(self.ui.actionHelp, QtCore.SIGNAL("triggered()"), self.ShowHelpWindow)
        QtCore.QObject.connect(self.ui.actionSettings, QtCore.SIGNAL("triggered()"), self.ShowConfigWindow)
        QtCore.QObject.connect(self.ui.actionAbout, QtCore.SIGNAL("triggered()"), About)
        QtCore.QObject.connect(self.ui.actionDonation, QtCore.SIGNAL("triggered()"), Donation)
        QtCore.QObject.connect(self.ui.actionShowRecordData, QtCore.SIGNAL("triggered()"), self.ShowRecordData)
        QtCore.QObject.connect(self.ui.actionExport, QtCore.SIGNAL("triggered()"), self.Export)

        self.setStyleSheet(data.UIStyle) #Add styles

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.bLast.setIcon(QIcon(data.app_path+"img/"+"arrow-left-double.png"))
            self.ui.bNext.setIcon(QIcon(data.app_path+"img/"+"arrow-right-double.png"))
            self.ui.bDelete.setIcon(QIcon(data.app_path+"img/"+"edittrash.png"))
            self.ui.bOpen.setIcon(QIcon(data.app_path+"img/"+"document-open-folder.png"))
            self.ui.bOpenDate.setIcon(QIcon(data.app_path+"img/"+"date.png"))
            self.ui.bZoomIn.setIcon(QIcon(data.app_path+"img/"+"zoom-in.png"))
            self.ui.bZoomOut.setIcon(QIcon(data.app_path+"img/"+"zoom-out.png"))
            self.ui.bEditNote.setIcon(QIcon(data.app_path+"img/"+"kwrite.png"))
            self.ui.bStatistics.setIcon(QIcon(data.app_path+"img/"+"klipper_dock.png"))

        #texts
        self.setWindowTitle(_("SleepAnalyser")+" ("+_("View")+")")
        self.ui.lblstate1.setText(_("Awake"))
        self.ui.lblstate2.setText(_("Light sleep"))
        self.ui.lblstate3.setText(_("Deep sleep"))
        self.ui.actionShowRecordData.setText(_("Additional information"))
        self.ui.actionHelp.setText(_("Help"))
        self.ui.actionAbout.setText(_("About"))
        self.ui.actionDonation.setText(_("Donation"))
        self.ui.actionSettings.setText(_("Settings"))
        self.ui.actionExport.setText(_("Export as image"))




    ###########################################################################
    ## Function:    UpdateScreenStart
    ## Description: Update window data on When window gets opened
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreenStart(self): ##(Class ViewWindow)
        pass





    ###########################################################################
    ## Function:    UpdateScreen
    ## Description: Update window data
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreen(self): ##(Class ViewWindow)
        day=strftime("%A %d. %B %Y", records.StartTime_tuple)
        d0=strftime("%H:%M", records.StartTime_tuple)
        d1=strftime("%H:%M", records.EndTime_tuple)
        diff_sec=records.EndTime_seconds-records.StartTime_seconds
        diff=time.strftime("%Hh %Mm", time.gmtime(diff_sec))
##        The following lines are disabled, currently i dont care if it could record for more than 24h
#        if(diff_sec>86400): d="1 day, " #longer than 24h
#        else: d=""
#        print day
#        self.ui.lblStart.setText(str(day))
        self.ui.lblStart.setText(QtGui.QApplication.translate("ViewWindow", day, None, QtGui.QApplication.UnicodeUTF8))
        self.ui.lblEnd.setText(d0+" - "+d1+"   ("+diff+")")

        self.ui.lblNote.setText(_("Note:")+" "+records.Note)

        if(len(data.arrRecordFiles)<2): #disable last/next buttons if only one record file in folder
            self.ui.bNext.setEnabled(False)
            self.ui.bLast.setEnabled(False)
        else:
            self.ui.bNext.setEnabled(True)
            self.ui.bLast.setEnabled(True)

        if(data.Xscale==0): self.ui.bZoomIn.setEnabled(False)
        else: self.ui.bZoomIn.setEnabled(True)
        if(data.Xscale==data.arrXscaleLen-1): self.ui.bZoomOut.setEnabled(False)
        else: self.ui.bZoomOut.setEnabled(True)



    ###########################################################################
    ## Function:    ShowRecord
    ## Description: Opens the record in the view window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowRecord(self): ##(Class ViewWindow)
        global app

        self.ui.lblStatus.setText("")
        self.ui.lblStart.setText("")
        self.ui.lblEnd.setText("")
        self.ui.lblNote.setText("")
        self.Canvas.hide()
        self.StatisticsCanvas.hide()

        self.ui.lblStatus.setText(_("Loading record..."))
        self.ui.lblStatus.show()
        app.processEvents() #force ui update

        r=LoadRecord()
        if(r==1): #file too long
            QtGui.QMessageBox.warning(self, _("SleepAnalyser"), _("The record is too long. The end will be cut off!"), QtGui.QMessageBox.Ok)
        elif(r==2): #file does not exist
            self.OpenRecord()
            return

        if(data.DeviceIsN900==True): #We are on a N900
            #It gives miss behavior on PC, so only do on N900
            self.UpdateScreen()
            app.processEvents() #force ui update

        self.ui.lblStatus.setText(_("Processing data..."))
        app.processEvents() #force ui update
        print "Processing data..."
        GenerateDerivation(True)

        self.ui.lblStatus.setText(_("Rendering data..."))
        app.processEvents() #force ui update
        print "Rendering data..."
        GenerateSleepPattern() #Recalculate sleep pattern curve (Note: This function is very time consuming)

        self.ui.lblStatus.setText(_("Generating statistic..."))
        app.processEvents() #force ui update
        print "Generating statistic..."

        GenerateStatisticData()

        t=max(records.EndTime_seconds, records.AlarmTime_seconds) - records.StartTime_seconds
        if(t<7*3600): data.Xscale=1 #if record time (or time start to alarm) is smaller than 7h, set to middle zoom level
        else: data.Xscale=2 #set to smallest zoom level

        self.ui.lblStatus.hide()
        ViewWindow.Canvas.show()  #ViewWindow instead of self, as it gets executed in RecordWindow
        ViewWindow.StatisticsCanvas.show() #ViewWindow instead of self, as it gets executed in RecordWindow
        Generate_and_show_GraphBitmap(self)
        GenerateStatisticsBitmap()

    #Note: On the PC, the lable texts disappear here again until the window looses the focus, reason is unknown
    #      But it works fine on the N900, so we ignore it




    ###########################################################################
    ## Function:    NextRecord
    ## Description: Button event: show next record
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def NextRecord(self): ##(Class ViewWindow)
        data.arrRecordFilesIndex=(data.arrRecordFilesIndex+1)  % len(data.arrRecordFiles)
        data.RecordFilename=data.arrRecordFiles[data.arrRecordFilesIndex]
        self.ShowRecord()




    ###########################################################################
    ## Function:    LastRecord
    ## Description: Button event: show previous record
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def LastRecord(self): ##(Class ViewWindow)
        data.arrRecordFilesIndex=(data.arrRecordFilesIndex-1)  % len(data.arrRecordFiles)
        data.RecordFilename=data.arrRecordFiles[data.arrRecordFilesIndex]
        self.ShowRecord()




    ###########################################################################
    ## Function:    ZoomIn
    ## Description: Button event: Zoom in
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ZoomIn(self): ##(Class ViewWindow)
        Zoom(self, +1, _("Zoom in..."))




    ###########################################################################
    ## Function:    ZoomOut
    ## Description: Button event: Zoom out
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ZoomOut(self): ##(Class ViewWindow)
        Zoom(self, -1, _("Zoom out..."))




    ###########################################################################
    ## Function:    DeleteRecord
    ## Description: Button event: Delete currently selected record, load last most recent record
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def DeleteRecord(self): ##(Class ViewWindow)
        self.ui.lblStatus.setText(_("Delete record..."))
        self.ui.lblStatus.show()
        reply = QtGui.QMessageBox.question(self, _("SleepAnalyser"), _("Are you sure you want to delete this record:")+"\n"+data.RecordFilename, QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
        if reply == QtGui.QMessageBox.Yes:
            print "Delete RecordFile: "+data.RecordFilename
            try:
                os.remove(data.RecordFolder+data.RecordFilename)
            except:
                print "Can not delete it. It might not exist"

            index=data.arrRecordFilesIndex
            LoadFileList()
            if(len(data.arrRecordFiles)>0):
#                data.RecordFilename=data.arrRecordFiles[l-1] #open last (youngest) record file
                data.arrRecordFilesIndex=index-1
                data.RecordFilename=data.arrRecordFiles[data.arrRecordFilesIndex]
            else: #no RecordFiles in RecordFolder, show "open file" dialog
                tmp=self.ShowFileDialog()
                print "## DeleteRecord", tmp
                if(tmp==""): #only continue if valid record file selected
                    self.closeEvent("") #no file selected, close
                    return
                tmp2=tmp.split("/")
                l=len(tmp2)
                data.RecordFilename=tmp2[l-1]
                data.RecordFolder=string.replace(tmp, data.RecordFilename, "") #get folder of selected file
                print "Set folder=", data.RecordFolder, ", file=", data.RecordFilename
            self.ShowRecord()
            self.UpdateScreen()
        self.ui.lblStatus.hide()
        self.StatisticsCanvas.show()




    ###########################################################################
    ## Function:    OpenRecord
    ## Description: Button event: Shows window to open a record
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def OpenRecord(self): ##(Class ViewWindow)

        self.ui.lblStart.setText("")
        self.ui.lblEnd.setText("")
        self.ui.lblNote.setText("")
        self.Canvas.hide()
        self.StatisticsCanvas.hide()
        self.ui.lblStatus.setText(_("Opening file dialog..."))
        self.ui.lblStatus.show()


        tmp=self.ShowFileDialog()
        print "## OpenRecord", tmp.toUtf8()
        if(tmp==""): #if user presses cancel tmp will be empty
            self.hide()
            StartWindow.show()
        f=str(tmp.toUtf8())
        f=f.decode("utf-8")
        if(not os.path.exists(f)): #only continue if valid record file selected
            self.ui.lblStatus.hide()
            print "file does not exist"
            return
        tmp2=tmp.split("/") #tmp includes whole path + file name!
        l=len(tmp2)
        data.RecordFilename=tmp2[l-1] #extract file name
        data.RecordFolder=string.replace(tmp, data.RecordFilename, "") #get folder of selected file
        print "Set folder=", data.RecordFolder.toUtf8(), ", file=", data.RecordFilename.toUtf8()
        LoadFileList()
        self.ShowRecord()
        self.ui.lblStatus.hide()
        ViewWindow.Canvas.show()
        ViewWindow.StatisticsCanvas.show()




    ###########################################################################
    ## Function:    ShowFileDialog
    ## Description: Shows the file dialog to select a record file
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowFileDialog(self): ##(Class ViewWindow)
        data.RecordFolder=string.replace(data.RecordFolder, "~", data.homedir) #workaround
        if(data.debug==True): print "Open file dialog in:", data.RecordFolder
        tmp= QFileDialog.getOpenFileName(None,_("Open SleepAnalyser record"), data.RecordFolder, _("Record Files")+" (*.csv)")
        print "Selected file:", tmp.toUtf8()
        return tmp






    ###########################################################################
    ## Function:    OpenDate
    ## Description: Button event: Shows the date selection window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def OpenDate(self): ##(Class ViewWindow)
        DateWindow.show()
        DateWindow.DateChanged()



    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: close window, show start window
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event): ##(Class ViewWindow)
        StartWindow.show()







    ###########################################################################
    ## Function:    EditNote
    ## Description: Button event: Opens a dialog to edit note
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def EditNote(self): ##(Class ViewWindow)
        r=QInputDialog.getText(None, _("SleepAnalyser"), _("Add a note to this record\n(leave empty for no note):"), QLineEdit.Normal, records.Note)
        if(r[1]==True): #OK pressed
            records.Note=unicode(r[0])
            self.ui.lblNote.setText(_("Note:")+" "+records.Note)
            UpdateFileHeader() #rewrite record file (only do when recording is stopped!)






    ###########################################################################
    ## Function:    ShowRecordData
    ## Description: Menu event: SHows the window with additional data of the currently selected record
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowRecordData(self): ##(Class ViewWindow)
        AdditionalInfoWindow.UpdateScreen()
        AdditionalInfoWindow.show()
        self.hide()




    ###########################################################################
    ## Function:    ShowConfigWindow
    ## Description: Menu event: Opens the config window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowConfigWindow(self): ##(Class ViewWindow)
        data.LastWindow="ViewWindow"
        ConfigWindow.UpdateScreen()
        ConfigWindow.show()
        self.hide()




    ###########################################################################
    ## Function:    ShowHelpWindow
    ## Description: Menu event: Show help window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowHelpWindow(self): ##(Class ViewWindow)
        data.LastWindow="ViewWindow"
        HelpWindow.load_data()
        HelpWindow.show()
        self.hide()





    ###########################################################################
    ## Function:    Export
    ## Description: Menu event: Opens a file selection dialog and exports the graph bitmap as a png file
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Export(self): ##(Class ViewWindow)
        filename=string.replace(data.RecordFilename, ".csv", "")
        tmp=QFileDialog.getSaveFileName(None,_("Export graph as"), data.defaultRecordFolder+filename, _("PNG-Image")+" (*.png)") #;;JPEG-Image (*.jpg)
        if(tmp!=""):
            if(tmp[-4:]!=".png"): tmp=tmp+".png" #add file extention if missing
            print "Export graph as :", tmp
            r=data.GraphBitMap.save(tmp, "png", -1)
            if(r==True): print "Export was successful"
            else: QtGui.QMessageBox.warning(self, _("SLeepAnalyser"), _("Failed to save graph as image:")+"\n"+tmp, QtGui.QMessageBox.Ok)
#        print "beep2"""
#        self.show()
#        HelpWindow.show()
#        ViewWindow.show()




#    def UpdateStatus(self, text): ##(Class ViewWindow)
#        self.ui.lblStatus.setText(text)
#        self.ui.lblStatus.show()
#        app.processEvents() #force ui update







###########################################################################
###########################################################################
## Function:    Class for wake up settings window
## Description: All functions needed for wake up option window
##                This window is used to set up the wake up option
##                It is only called from the set recording window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmWakeUpSettingsWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_WakeUpSettingsWindow()
        self.ui.setupUi(self)

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.bSet_Music.setIcon(QIcon(data.app_path+"img/"+"music_red.png"))
            self.ui.bPlay.setIcon(QIcon(data.app_path+"img/"+"media-playback-start.png"))
            self.ui.bSelect_Song.setIcon(QIcon(data.app_path+"img/"+"document-open.png"))
            self.ui.bSelect_Song_folder.setIcon(QIcon(data.app_path+"img/"+"document-open-folder.png"))

        QtCore.QObject.connect(self.ui.bSet_Music, QtCore.SIGNAL("clicked()"), self.Set_Music)
        QtCore.QObject.connect(self.ui.bSelect_Song, QtCore.SIGNAL("clicked()"), self.Select_Song)
        QtCore.QObject.connect(self.ui.bSelect_Song_folder, QtCore.SIGNAL("clicked()"), self.Select_Song_folder)
        QtCore.QObject.connect(self.ui.bPlay, QtCore.SIGNAL("clicked()"), self.Play)
        
        QtCore.QObject.connect(self.ui.SlrVolume,SIGNAL("valueChanged(int)"),self.VolumeSliderMoved)
#        QtCore.QObject.connect(self.ui.SlrDuration,SIGNAL("valueChanged(int)"),self.DurationSliderMoved)

        self.setStyleSheet(data.UIStyle) #Add styles

        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Wake up option")+")")
        self.ui.bSet_Music.setText(_("Set Wake up music"))
        self.ui.lblSong.setText(_("Song:"))
        self.ui.lblDuration.setText(_("Duration:"))
        self.ui.lblVolume.setText(_("Volume:"))
        self.ui.lblNote.setText(_("The WakeUp song will be played once with the set volume when the alarm gets triggered. The real alarm gets delayed for the duration of the song."))




    ###########################################################################
    ## Function:    UpdateScreen
    ## Description: Update window data
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreen(self): ## Class Wake Up SettingsWindow
        self.ui.lblVolume.setText(_("Volume:")+" "+str(data.WakeUp_Song_Volume)+"%")
        self.ui.SlrVolume.setValue(int(data.WakeUp_Song_Volume))

        if(data.WakeUp_Song!=""):
            tmp2=data.WakeUp_Song.split("/")
            l=len(tmp2)
            txt=tmp2[l-1]
        else:
            txt=_("(not set)")
        txt=QtGui.QApplication.translate("", txt, None, QtGui.QApplication.UnicodeUTF8)
        if(os.path.isfile(str(data.WakeUp_Song))):
            txt=_("Song file:")+" "+txt
            self.ui.lblDuration.setText(_("Duration:")+" "+time.strftime("%Hh %Mm %Ss", time.gmtime(data.WakeUp_Song_Duration)))
        else:
            txt=_("Song folder:")+" "+txt
            self.ui.lblDuration.setText(str(data.WakeUp_Song_Duration)+" "+_("files in folder")) #abuse this variable for file count
        self.ui.lblSong.setText(txt)

        





    ###########################################################################
    ## Function:    Set_Music
    ## Description: Enables the option and closes the screen
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Set_Music(self):## Class Wake Up SettingsWindow
        if(os.path.exists(str(data.WakeUp_Song))):
            data.Use_WakeUp_Song=True
            SetRecordingWindow.ui.cbWakeUpSongEnable.setChecked(True)
        SetRecordingWindow.UpdateScreen()
        SetRecordingWindow.show()
        WakeUpSettingsWindow.hide()





    ###########################################################################
    ## Function:    Select_Song
    ## Description: Opens the file selection dialog and sets the song for the wke up option
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Select_Song(self):## Class Wake Up SettingsWindow
        f=QtGui.QApplication.translate("", data.WakeUp_Song, None, QtGui.QApplication.UnicodeUTF8)
        f= QFileDialog.getOpenFileName(None,_("Select song"), f, _("Music file")+" ("+data.song_file_types+")")
        f=f.toUtf8()
        tmp=str(f)       
        tmp2=tmp.decode("utf-8")        
            
        if(CheckFileValidity(tmp)==0): #ceck for illegal characters in file/folder path
            return
        elif(tmp2!="" and os.path.exists(tmp2)):
            data.WakeUp_Song=f
            print "Selected file:", f
            data.WakeUp_Song_Duration=GetSongLength(data.WakeUp_Song)
        else:
            print "No song selected or not valid, keep last one"
        self.UpdateScreen()





    ###########################################################################
    ## Function:    Select_Song_folder
    ## Description: Button event:Opens folderdialog to select song folder
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Select_Song_folder(self):## Class Wake up SettingsWindow
        f=QtGui.QApplication.translate("", data.WakeUp_Song, None, QtGui.QApplication.UnicodeUTF8)
        f= QFileDialog.getExistingDirectory(None,_("Select song folder"), f)
        f=f.toUtf8()
        tmp=str(f)            
        tmp2=tmp.decode("utf-8")        
            
        if(CheckFileValidity(tmp)==0): #ceck for illegal characters in file/folder path
            return
        elif(tmp2!="" and os.path.exists(tmp2)):
            arr=getFileList(f,  data.song_file_types)
            filecount=len(arr)
            if(filecount<1):
                print "No sound file in this folder"
                QtGui.QMessageBox.warning(self, _("SleepAnalyser"), _("No sound files in this folder.\nPlease first select another folder!"), QtGui.QMessageBox.Ok)
                return            
            data.WakeUp_Song=f
            data.WakeUp_Song_Duration=filecount #abuse this vairable for number of files
            print "Selected folder:", f,  ", total",  str(filecount),  "song files"
        else:
            print "No folder selected or not valid, keep last one"
        self.UpdateScreen()
        
        
        
        
        
    ###########################################################################
    ## Function:    Play
    ## Description: Button event: Play selected song
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Play(self):## Class Wake Up SettingsWindow
        Play(data.WakeUp_Song,  data.WakeUp_Song_Volume)




    ###########################################################################
    ## Function:    VolumeSliderMoved
    ## Description: Slider event: Update new slider value
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def VolumeSliderMoved(self):## Class Wake Up SettingsWindow
        x=self.ui.SlrVolume.value()
        data.WakeUp_Song_Volume=x
        self.UpdateScreen()



    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: close window, show recording settings window
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event): ## Class Wake Up SettingsWindow
#        self.Set_Music()
        SetRecordingWindow.UpdateScreen()
        SetRecordingWindow.show()
        WakeUpSettingsWindow.hide()











###########################################################################
###########################################################################
## Function:    Class for lucid dream settings window
## Description: All functions needed for lucid dream option window
##                This window is used to set up the lucid dream option
##                It is only called from the set recording window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmLucidDreamSettingsWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_LucidDreamSettingsWindow()
        self.ui.setupUi(self)

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.bSet_Music.setIcon(QIcon(data.app_path+"img/"+"music_green.png"))
            self.ui.bPlay.setIcon(QIcon(data.app_path+"img/"+"media-playback-start.png"))
            self.ui.bSelect_Song.setIcon(QIcon(data.app_path+"img/"+"document-open.png"))
            self.ui.bSelect_Song_folder.setIcon(QIcon(data.app_path+"img/"+"document-open-folder.png"))

        QtCore.QObject.connect(self.ui.bSet_Music, QtCore.SIGNAL("clicked()"), self.Set_Music)
        QtCore.QObject.connect(self.ui.bSelect_Song, QtCore.SIGNAL("clicked()"), self.Select_Song)
        QtCore.QObject.connect(self.ui.bSelect_Song_folder, QtCore.SIGNAL("clicked()"), self.Select_Song_folder)
        QtCore.QObject.connect(self.ui.bPlay, QtCore.SIGNAL("clicked()"), self.Play)

        QtCore.QObject.connect(self.ui.SlrVolume,SIGNAL("valueChanged(int)"),self.VolumeSliderMoved)
        QtCore.QObject.connect(self.ui.SlrDelay,SIGNAL("valueChanged(int)"),self.DelaySliderMoved)

        self.setStyleSheet(data.UIStyle) #Add styles

        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Lucid dream option")+")")
        self.ui.bSet_Music.setText(_("Set lucid dream music"))
        self.ui.lblSong.setText(_("Song:"))
        self.ui.lblDelay.setText(_("Delay:"))
        self.ui.lblVolume.setText(_("Volume:"))
        self.ui.lblDuration.setText(_("Duration:"))
        self.ui.lblNote.setText(_("The lucid dream song will be played once with the set volume after the set delay time in light or deep sleep."))




    ###########################################################################
    ## Function:    UpdateScreen
    ## Description: Update window data
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreen(self):## Class Lucid Dream SettingsWindow
        self.ui.lblVolume.setText(_("Volume:")+" "+str(data.Lucid_Dream_Song_Volume)+"%")
        self.ui.SlrVolume.setValue(int(data.Lucid_Dream_Song_Volume))

        m=int(math.floor(data.Lucid_Dream_Song_Delay/60))
        self.ui.lblDelay.setText(_("Delay:")+" "+str(m)+" "+ _("minutes"))
        self.ui.SlrDelay.setValue(int(m))

        if(data.Lucid_Dream_Song!=""):
            tmp2=data.Lucid_Dream_Song.split("/")
            l=len(tmp2)
            txt=tmp2[l-1]
        else:
            txt=_("(not set)")
        txt=QtGui.QApplication.translate("", txt, None, QtGui.QApplication.UnicodeUTF8)
        if(os.path.isfile(str(data.Lucid_Dream_Song))):
            txt=_("Song file:")+" "+txt
            self.ui.lblDuration.setText(_("Duration:")+" "+time.strftime("%Hh %Mm %Ss", time.gmtime(data.Lucid_Dream_Song_Duration)))
        else:
            txt=_("Song folder:")+" "+txt      
            self.ui.lblDuration.setText(str(data.Lucid_Dream_Song_Duration)+" "+_("files in folder")) #abuse this variable for file count      
        self.ui.lblSong.setText(txt)





    ###########################################################################
    ## Function:    Set_Music
    ## Description: Button event: CLose the window, open set recording window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Set_Music(self):## Class Lucid Dream SettingsWindow
        if(os.path.exists(str(data.Lucid_Dream_Song))):
            data.Use_Lucid_Dream_Song=True
            SetRecordingWindow.ui.cbLucidDreamSongEnable.setChecked(True)
        SetRecordingWindow.UpdateScreen()
        SetRecordingWindow.show()
        LucidDreamSettingsWindow.hide()




    ###########################################################################
    ## Function:    Select_Song
    ## Description: Button event:Opens file dialog to select song
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Select_Song(self):## Class Lucid Dream SettingsWindow
        f=QtGui.QApplication.translate("", data.Lucid_Dream_Song, None, QtGui.QApplication.UnicodeUTF8)
        f= QFileDialog.getOpenFileName(None,_("Select song"), f, _("Music file")+" ("+data.song_file_types+")")
        f=f.toUtf8()       
        tmp=str(f)            
        tmp2=tmp.decode("utf-8")        
            
        if(CheckFileValidity(tmp)==0): #ceck for illegal characters in file/folder path
            return
        elif(tmp2!="" and os.path.exists(tmp2)):
            data.Lucid_Dream_Song=f
            print "Selected file:", f
            data.Lucid_Dream_Song_Duration=GetSongLength(data.Lucid_Dream_Song)
        else:
            print "No song selected or not valid, keep last one"
        self.UpdateScreen()




    ###########################################################################
    ## Function:    Select_Song_folder
    ## Description: Button event: Opens folderdialog to select song folder
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Select_Song_folder(self):## Class Lucid Dream SettingsWindow
        f=QtGui.QApplication.translate("", data.Lucid_Dream_Song, None, QtGui.QApplication.UnicodeUTF8)
        f= QFileDialog.getExistingDirectory(None,_("Select song folder"), f)
        f=f.toUtf8()
        tmp=str(f)   
        tmp2=tmp.decode("utf-8")
        if(CheckFileValidity(tmp)==0): #ceck for illegal characters in file/folder path
            return
        elif(tmp2!="" and os.path.exists(tmp2)):
            arr=getFileList(f,  data.song_file_types)
            filecount=len(arr)
            if(filecount<1):
                print "No sound file in this folder"
                QtGui.QMessageBox.warning(self, _("SleepAnalyser"), _("No sound files in this folder.\nPlease first select another folder!"), QtGui.QMessageBox.Ok)
                return            
            data.Lucid_Dream_Song=f
            data.Lucid_Dream_Song_Duration=filecount
            print "Selected folder:", f,  ", total",  str(filecount),  "song files"
        else:
            print "No folder selected or not valid, keep last one"
        self.UpdateScreen()
        
        
        
        
        
    ###########################################################################
    ## Function:    Play
    ## Description: Button event: Play selected song
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Play(self):## Class Lucid Dream SettingsWindow
        Play(data.Lucid_Dream_Song,  data.Lucid_Dream_Song_Volume)





    ###########################################################################
    ## Function:    VolumeSliderMoved
    ## Description: Slider event: Update new slider value
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def VolumeSliderMoved(self):## Class Lucid Dream SettingsWindow
        x=self.ui.SlrVolume.value()
        data.Lucid_Dream_Song_Volume=x
        self.UpdateScreen()




    ###########################################################################
    ## Function:    DelaySliderMoved
    ## Description: Slider event: Update new slider value
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def DelaySliderMoved(self):## Class Lucid Dream SettingsWindow
        x=self.ui.SlrDelay.value()
        data.Lucid_Dream_Song_Delay=x*60
        self.UpdateScreen()



    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: close window, show recording settings window
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event): ## Class Lucid Dream SettingsWindow
#        self.Set_Music()
        SetRecordingWindow.UpdateScreen()
        SetRecordingWindow.show()
        LucidDreamSettingsWindow.hide()














###########################################################################
###########################################################################
## Function:    Class for recording setup window
## Description: All functions needed for recording setup window
##                This window is used to set up all recording option
##                It is only called from the set start window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmSetRecordingWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_SetRecordingWindow()
        self.ui.setupUi(self)

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.lblIcon.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"SleepAnalyser.png"))
            self.ui.lblLucid_Dream_Icon.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"music_green.png"))
            self.ui.lblAlarm_Time_Icon.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"SleepAnalyser.png"))
            self.ui.lblWakeUp_Icon.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"music_red.png"))
            self.ui.bStart.setIcon(QIcon(data.app_path+"img/"+"krec.png"))

        QtCore.QObject.connect(self.ui.bSet_Lucid_Dream_Options, QtCore.SIGNAL("clicked()"), self.Set_Lucid_Dream_Options)
        QtCore.QObject.connect(self.ui.cbLucidDreamSongEnable, QtCore.SIGNAL("toggled(bool)"), self.Lucid_DreamSongEnable)

        QtCore.QObject.connect(self.ui.bSet_Wake_Up_Options, QtCore.SIGNAL("clicked()"), self.Set_Wake_Up_Options)
        QtCore.QObject.connect(self.ui.cbWakeUpSongEnable, QtCore.SIGNAL("toggled(bool)"), self.WakeUpSongEnable)

        QtCore.QObject.connect(self.ui.bSet_Alarm, QtCore.SIGNAL("clicked()"), self.SetAlarm)
        QtCore.QObject.connect(self.ui.cbAlarmEnable, QtCore.SIGNAL("toggled(bool)"), self.AlarmEnable)

        QtCore.QObject.connect(self.ui.bStart, QtCore.SIGNAL("clicked()"), self.Start)

        self.setStyleSheet(data.UIStyle) #Add styles

        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Setup recording")+")")
        self.ui.bStart.setText(_("Start"))
        self.ui.lblLucid_Dream.setText(_("Lucid dream option"))
        self.ui.lblLucid_Dream_Volume.setText(_("Volume:"))
        self.ui.lblAlarmTime2.setText(_("Alarm option"))
        self.ui.bSet_Alarm.setText("...")
        self.ui.lblLucid_Dream_Song.setText(_("Song:"))
        self.ui.bSet_Lucid_Dream_Options.setText("...")
        self.ui.lblAlarmTime.setText(_("Alarm time:"))
        self.ui.lblWakeUp_Song.setText(_("Song:"))
        self.ui.lblWakeUp.setText(_("Wake up option"))
        self.ui.bSet_Wake_Up_Options.setText("...")
        self.ui.lblWakeUp_Volume.setText(_("Volume:"))
        self.ui.lblAlarmType.setText(_("Alarm type:"))






    ###########################################################################
    ## Function:    UpdateScreenStart
    ## Description: Update window data on start
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreenStart(self):
        if(data.Lucid_Dream_Song=="" or not os.path.exists(str(data.Lucid_Dream_Song))):
            data.Lucid_Dream_Song_Duration=0
            self.ui.cbLucidDreamSongEnable.setChecked(False)
        else:
            if(data.Use_Lucid_Dream_Song==True):
                self.ui.cbLucidDreamSongEnable.setChecked(True)
            else:
                self.ui.cbLucidDreamSongEnable.setChecked(False)
                
        if(data.WakeUp_Song=="" or not os.path.exists(str(data.WakeUp_Song))):
            data.WakeUp_Song_Duration=0
            self.ui.cbWakeUpSongEnable.setChecked(False)
        else:
            if(data.Use_WakeUp_Song==True):
                self.ui.cbWakeUpSongEnable.setChecked(True)
            else:
                self.ui.cbWakeUpSongEnable.setChecked(False)
                
            if(os.path.isfile(str(data.WakeUp_Song))):
                txt=_("Song file:")
            else:
                txt=_("Song folder:")
            self.ui.lblWakeUp_Song.setText(txt)
                

        if(data.UseAlarm==True): self.ui.cbAlarmEnable.setChecked(True)
        else: self.ui.cbAlarmEnable.setChecked(False)




    ###########################################################################
    ## Function:    UpdateScreen
    ## Description: Update window data
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreen(self): ##(Class SetRecordingWindow)
        if(data.Use_Lucid_Dream_Song==True):
            self.ui.lblLucid_Dream_Song.setEnabled(True)
            self.ui.lblLucid_Dream_Volume.setEnabled(True)
        else:
            self.ui.lblLucid_Dream_Song.setEnabled(False)
            self.ui.lblLucid_Dream_Volume.setEnabled(False)

        if(data.Lucid_Dream_Song!=""):
            tmp2=data.Lucid_Dream_Song.split("/")
            l=len(tmp2)
            txt=tmp2[l-1]                            
            if(os.path.isfile(str(data.Lucid_Dream_Song))):
                txt2=_("Song file:")
                duration=_("Duration:")+" "+time.strftime("%Hh %Mm %Ss", time.gmtime(data.Lucid_Dream_Song_Duration))
            else:
                txt2=_("Song folder:")
                duration=str(data.Lucid_Dream_Song_Duration)+" "+_("files in folder") #abuse this variable for file count
        else:
            txt2=_("Song file:")
            txt=_("(not set)")
            duration=""

        self.ui.lblLucid_Dream_Song.setText(txt2+" "+QtGui.QApplication.translate("", txt, None, QtGui.QApplication.UnicodeUTF8))
        m=int(math.floor(data.Lucid_Dream_Song_Delay/60))
        txt=_("start after $ minutes in light or deep sleep")
        txt=string.replace(txt, "$", str(m)) + ", "+duration
        self.ui.lblLucid_Dream_Volume.setText(_("Volume:")+" "+str(data.Lucid_Dream_Song_Volume)+"%, "+ txt)

        if(data.WakeUp_Song!=""):
            tmp2=data.WakeUp_Song.split("/")
            l=len(tmp2)
            txt=tmp2[l-1]                            
            if(os.path.isfile(str(data.WakeUp_Song))):
                txt2=_("Song file:")
                duration=_("Duration:")+" "+time.strftime("%Hh %Mm %Ss", time.gmtime(data.WakeUp_Song_Duration))
            else:
                txt2=_("Song folder:")
                duration=str(data.Lucid_Dream_Song_Duration)+" "+_("files in folder") #abuse this variable for file count
        else:
            txt2=_("Song file:")
            txt=_("(not set)")
            duration=""
            
        self.ui.lblWakeUp_Song.setText(txt2+" "+QtGui.QApplication.translate("", txt, None, QtGui.QApplication.UnicodeUTF8))
        self.ui.lblWakeUp_Volume.setText(_("Volume:")+" "+str(data.WakeUp_Song_Volume)+"%, "+duration)

        ########################
        if(data.UseAlarm==True):
            self.ui.lblAlarmTime.setEnabled(True)
            self.ui.lblAlarmTime.setEnabled(True)
            self.ui.cbWakeUpSongEnable.setEnabled(True)
            self.ui.bSet_Wake_Up_Options.setEnabled(True)
            self.ui.lblWakeUp_Song.setEnabled(True)
            self.ui.lblWakeUp_Icon.setEnabled(True)
            self.ui.lblWakeUp.setEnabled(True)
            self.ui.lblAlarmType.setEnabled(True)
            if(data.Use_WakeUp_Song==True):
                self.ui.lblWakeUp_Song.setEnabled(True)
                self.ui.lblWakeUp_Volume.setEnabled(True)
            else:
                self.ui.lblWakeUp_Song.setEnabled(False)
                self.ui.lblWakeUp_Volume.setEnabled(False)

        else:
            self.ui.lblAlarmTime.setEnabled(False)
            self.ui.lblAlarmTime.setEnabled(False)
            self.ui.cbWakeUpSongEnable.setEnabled(False)
            self.ui.bSet_Wake_Up_Options.setEnabled(False)
            self.ui.lblWakeUp_Icon.setEnabled(False)
            self.ui.lblWakeUp.setEnabled(False)
            self.ui.lblWakeUp_Song.setEnabled(False)
            self.ui.lblWakeUp_Volume.setEnabled(False)
            self.ui.lblAlarmType.setEnabled(False)

        if(data.AlarmHour==-1 or data.AlarmMinute==-1):
            data.AlarmHour=7
            data.AlarmMinute=0
            data.AlarmTimeWindow=30
        al0=int(data.AlarmHour)*60+int(data.AlarmMinute)-data.AlarmTimeWindow
        h0=int(math.floor(al0/60))
        m0=al0-(h0*60)
        m1=data.AlarmMinute
        if(m0<10): m0="0"+str(m0)
        if(m1<10): m1="0"+str(m1)
        txt=str(h0)+":"+str(m0) + "-" + str(data.AlarmHour)+":"+str(m1)
        self.ui.lblAlarmTime.setText(_("Alarm time:")+" "+ txt)

        if(data.UseAlternativeAlarm==False):
            self.ui.lblAlarmType.setText(_("Alarm type:")+" "+ _("Normal"))
        else:
            self.ui.lblAlarmType.setText(_("Alarm type:")+" "+_("Alternative alarm")+" "+str(data.AlternativeAlarm))



    ###########################################################################
    ## Function:    Lucid_DreamSongEnable
    ## Description: Button event: enable/disable lucid dream option
    ## Parameters:  self, check box state
    ## Returns:     none
    ###########################################################################
    def Lucid_DreamSongEnable(self, state): ##(Class SetRecordingWindow)
        if(state==True):
            if(not os.path.exists(str(data.Lucid_Dream_Song))):
                print "Sound file/folder not valid:", data.Lucid_Dream_Song
                QtGui.QMessageBox.warning(self, _("SleepAnalyser"), _("Please first select a valid song file or folder!"), QtGui.QMessageBox.Ok)
                self.ui.cbLucidDreamSongEnable.setChecked(False)
                return
            data.Use_Lucid_Dream_Song=True
        else:
            data.Use_Lucid_Dream_Song=False
        self.UpdateScreen()




    ###########################################################################
    ## Function:    Set_Lucid_Dream_Options
    ## Description: Button event: show lucid dream option window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Set_Lucid_Dream_Options(self): ##(Class SetRecordingWindow)
        LucidDreamSettingsWindow.UpdateScreen()
        LucidDreamSettingsWindow.show()
        SetRecordingWindow.hide()



    ###########################################################################
    ## Function:    WakeUpSongEnable
    ## Description: Button event: enable/disable wake up option
    ## Parameters:  self, check box state
    ## Returns:     none
    ###########################################################################
    def WakeUpSongEnable(self, state): ##(Class SetRecordingWindow)
        if(state==True):
            if(not os.path.exists(str(data.WakeUp_Song))):
                print "Sound file/folder not valid:", data.WakeUp_Song
                QtGui.QMessageBox.warning(self, _("SleepAnalyser"), _("Please first select a valid song file or folder!"), QtGui.QMessageBox.Ok)
                self.ui.cbWakeUpSongEnable.setChecked(False)
                return
            data.Use_WakeUp_Song=True
        else:
            data.Use_WakeUp_Song=False
        self.UpdateScreen()




    ###########################################################################
    ## Function:    Set_Wake_Up_Options
    ## Description: Button event: show wake up option window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Set_Wake_Up_Options(self): ##(Class SetRecordingWindow)
        WakeUpSettingsWindow.UpdateScreen()
        WakeUpSettingsWindow.show()
        SetRecordingWindow.hide()




    ###########################################################################
    ## Function:    AlarmEnable
    ## Description: Button event: enable/disable alarm option
    ## Parameters:  self, check box state
    ## Returns:     none
    ###########################################################################
    def AlarmEnable(self, state): ##(Class SetRecordingWindow)
        if(state==True):
            data.UseAlarm=True
        else:
            data.UseAlarm=False
        self.UpdateScreen()




    ###########################################################################
    ## Function:    SetAlarm
    ## Description: Button event: Show alarm option window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def SetAlarm(self): ##(Class SetRecordingWindow)
        AlarmSettingsWindow.UpdateScreen()
        AlarmSettingsWindow.show()
        SetRecordingWindow.hide()






    ###########################################################################
    ## Function:    Start
    ## Description: Button event: Start the recording
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Start(self): ##(Class SetRecordingWindow)
        InitRecord() #clear all old data, initialise next measurement


        records.Use_Lucid_Dream_Song = data.Use_Lucid_Dream_Song  
        if(data.Use_Lucid_Dream_Song==True):
            records.Lucid_Dream_Song_Delay = data.Lucid_Dream_Song_Delay
            records.Lucid_Dream_Song_Volume = data.Lucid_Dream_Song_Volume     
            if(os.path.isfile(str(data.Lucid_Dream_Song))): 
                records.Lucid_Dream_Song = data.Lucid_Dream_Song
                records.Lucid_Dream_Song_Duration = data.Lucid_Dream_Song_Duration
            else: #is folder, select a random file
                arr=getFileList(data.Lucid_Dream_Song,  data.song_file_types)
                i=int(random.uniform(0,  len(arr)-1))
                records.Lucid_Dream_Song = data.Lucid_Dream_Song+"/"+arr[i]
                records.Lucid_Dream_Song_Duration = GetSongLength(records.Lucid_Dream_Song)            
            print "Lucid dream song:",  records.Lucid_Dream_Song,  ", duration:",  records.Lucid_Dream_Song_Duration
    

        if(data.UseAlarm==False):#its not possible to have a wake up song without alarm
            records.Use_WakeUp_Song=False
        else:
            records.Use_WakeUp_Song = data.Use_WakeUp_Song
            if(data.Use_WakeUp_Song==True):
                records. WakeUp_Song_Volume= data.WakeUp_Song_Volume                
                if(os.path.isfile(str(data.WakeUp_Song))): 
                    records.WakeUp_Song = data.WakeUp_Song
                    records.WakeUp_Song_Duration = data.WakeUp_Song_Duration
                else: #is folder, select a random file
                    arr=getFileList(data.WakeUp_Song,  data.song_file_types)   
                    i=int(random.uniform(0,  len(arr)-1))
                    records.WakeUp_Song = data.WakeUp_Song+"/"+arr[i]
                    records.WakeUp_Song_Duration = GetSongLength(records.WakeUp_Song)            
                print "Wake Up song:",  records.WakeUp_Song,  ", duration:",  records.WakeUp_Song_Duration
        

        if(data.UseAlarm==True): #Use alarm option, write alarm data into record
            try:
                now=int(strftime("%H", time.localtime()))*3600+int(strftime("%M", time.localtime()))*60
                al=int(data.AlarmHour)*3600+int(data.AlarmMinute)*60
                alarmdiff=al-now
                if(alarmdiff<0):  #set to tomorrow if time already is elapsed today
                    alarmdiff=alarmdiff+86400

                records.AlarmTimeWindow_minutes=data.AlarmTimeWindow

                records.AlarmTime_seconds=int((records.StartTime_seconds+alarmdiff)/60)*60 #set seconds to zero
                records.AlarmTime_tuple=Seconds2Tuple(records.AlarmTime_seconds)

                records.AlarmTimeWindowEnd_tuple=records.AlarmTime_tuple
                records.AlarmTimeWindowEnd_seconds=records.AlarmTime_seconds

                records.AlarmTimeWindowStart_seconds=records.AlarmTimeWindowEnd_seconds-records.AlarmTimeWindow_minutes*60
                records.AlarmTimeWindowStart_tuple=Seconds2Tuple(records.AlarmTimeWindowStart_seconds)

            except: #If nothing selected (workaround)
                print "ERROR: Selected time not valid!"

        records.UseAlternativeAlarm= data.UseAlternativeAlarm
        records.AlternativeAlarm=data.AlternativeAlarm

        data.QuickMode=False #run in normal mode

        print "*", strftime("%x %X"), "SetAlarm(Class AlarmTimeWindow) => DeleteAlarm" #for log file
        DeleteAlarm() #delete old alarm if existing
        delete_all_Alarms() #Check if there are old alarms. If SleepAnalyser crashed, they might still be set

        if(data.UseAlarm==True): #Use alarm
            records.AlarmIsSet=True
            print "*", strftime("%x %X"), "SetAlarm(Class AlarmTimeWindow) => SetAlarm" #for log file
            SetAlarm() #set alarm for the last possible time


#            print_list_of_all_alarms()
#            delete_all_Alarms()
#            print_list_of_all_alarms()

#            print "------------------------------------"
#
#            for i in range(0, 1000):
#                print "######### TEST LOOP", i
#                print "records.AlarmTime_tuple:", records.AlarmTime_tuple
#                DeleteAlarm()
#                SetAlarm()
#            print_list_of_all_alarms()
#            delete_all_Alarms()
#            exit()


        self.SetConfig()

        InitMeasurement()

        RecordWindow.show()
        SetRecordingWindow.hide()
        RecordWindow.UpdateScreenStart()





    ###########################################################################
    ## Function:    SetConfig
    ## Description: Updates the config values
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def SetConfig(self): ##(Class SetRecordingWindow)
        #update config values
        config.set('main', 'Use_Lucid_Dream_Song', str(data.Use_Lucid_Dream_Song))
        config.set('main', 'Lucid_Dream_Song', str(data.Lucid_Dream_Song))
        config.set('main', 'Lucid_Dream_Song_Volume', str(data.Lucid_Dream_Song_Volume))
        config.set('main', 'Lucid_Dream_Song_Delay', str(data.Lucid_Dream_Song_Delay))
        config.set('main', 'Lucid_Dream_Song_Duration', str(data.Lucid_Dream_Song_Duration))

        config.set('main', 'Use_WakeUp_Song', str(data.Use_WakeUp_Song))
        config.set('main', 'WakeUp_Song', str(data.WakeUp_Song))
        config.set('main', 'WakeUp_Song_Volume', str(data.WakeUp_Song_Volume))
        config.set('main', 'WakeUp_Song_Duration', str(data.WakeUp_Song_Duration))

        config.set('main', 'UseAlarm', str(data.UseAlarm))
        config.set('main', 'AlarmHour', str(data.AlarmHour))
        config.set('main', 'AlarmMinute', str(data.AlarmMinute))
        config.set('main', 'AlarmTimeWindow', str(data.AlarmTimeWindow))

        config.set('main', 'UseAlternativeAlarm', str(data.UseAlternativeAlarm))

        WriteConfig() #good time to save config





    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: close window, show start window
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event): ##(Class SetRecordingWindow)
        self.SetConfig()
        StartWindow.show()












###########################################################################
###########################################################################
## Function:    Class for help window
## Description: All functions needed for additional help window
##                This window is used to show the help file
##                It is only called from the start, recording and view window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmHelpWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_HelpWindow()
        self.ui.setupUi(self)

        self.setStyleSheet(data.UIStyle) #Add styles




    ###########################################################################
    ## Function:    load_data
    ## Description: Load help file into browser window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def load_data(self):
        filename=data.app_path+"help_"+data.language+".htm"
        if(os.path.exists(str(filename))):
            print "Open help file:", filename
        else:
            print "Help file", filename, "does not exist"
            filename=data.app_path+"help_en.htm"
            print "Open default help file:", filename

        file = open(filename, 'r')
        txt=""
        for line in file: txt=txt+line+"\n"
        file.close()
        txt=QtGui.QApplication.translate("", txt, None, QtGui.QApplication.UnicodeUTF8)
        self.ui.textBrowser.setHtml(txt)

        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Help")+")")





    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: close window, show start or view window
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event): ##(Class HelpWindow)
        self.close()
        if(data.LastWindow=="ViewWindow"): ViewWindow.show()

        else: StartWindow.show()


















###########################################################################
###########################################################################
## Function:    Class for configuration window
## Description: All functions needed for configuration window
##                This window is used to set some persistent, global parameters
##                It is only called from the start and view window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmConfigWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_ConfigWindow()
        self.ui.setupUi(self)
#        self.ui.lblIcon.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"SleepAnalyser.png"))
        self.ui.imgYscale_Movement.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"Yscale_Movement.png"))
        self.ui.imgYscale_Sleep_Pattern.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"Yscale_Sleep_Pattern.png"))
        self.ui.imgYoffset_Sleep_Pattern.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"Yoffset_Sleep_Pattern.png"))
        self.ui.imgDelayStart.setPixmap(QtGui.QPixmap(data.app_path+"img/"+"DelayStart.png"))

        self.ui.bYscale_Movement_Reset.setIcon(QIcon(data.app_path+"img/"+"edit-undo.png"))
        self.ui.bYscale_Sleep_Pattern_Reset.setIcon(QIcon(data.app_path+"img/"+"edit-undo.png"))
        self.ui.bYoffset_Sleep_Pattern_Reset.setIcon(QIcon(data.app_path+"img/"+"edit-undo.png"))
        self.ui.bDelayStart_Reset.setIcon(QIcon(data.app_path+"img/"+"edit-undo.png"))

        #signals
        QtCore.QObject.connect(self.ui.bEditUserName, QtCore.SIGNAL("clicked()"), self.EditUserName)
        QtCore.QObject.connect(self.ui.SlrDelayStart,SIGNAL("valueChanged(int)"),self.DelaySliderMoved)
        QtCore.QObject.connect(self.ui.SlrYscale_Movement,SIGNAL("valueChanged(int)"),self.YscaleSliderMoved_Movement)
        QtCore.QObject.connect(self.ui.SlrYscale_Sleep_Pattern,SIGNAL("valueChanged(int)"),self.YscaleSliderMoved_Sleep_Pattern)
        QtCore.QObject.connect(self.ui.SlrYoffset_Sleep_Pattern,SIGNAL("valueChanged(int)"),self.YoffsetSliderMoved_Sleep_Pattern)
        QtCore.QObject.connect(self.ui.bYscale_Movement_Reset, QtCore.SIGNAL("clicked()"), self.Yscale_Movement_Reset)
        QtCore.QObject.connect(self.ui.bYscale_Sleep_Pattern_Reset, QtCore.SIGNAL("clicked()"), self.Yscale_Sleep_Pattern_Reset)
        QtCore.QObject.connect(self.ui.bYoffset_Sleep_Pattern_Reset, QtCore.SIGNAL("clicked()"), self.Yoffset_Sleep_Pattern_Reset)
        QtCore.QObject.connect(self.ui.bDelayStart_Reset, QtCore.SIGNAL("clicked()"), self.DelayStart_Reset)

        QtCore.QObject.connect(self.ui.bSave_Preset_1, QtCore.SIGNAL("clicked()"), self.SavePreset_1)
        QtCore.QObject.connect(self.ui.bLoad_Preset_1, QtCore.SIGNAL("clicked()"), self.LoadPreset_1)
        QtCore.QObject.connect(self.ui.bRename_Preset_1, QtCore.SIGNAL("clicked()"), self.RenamePreset_1)
        QtCore.QObject.connect(self.ui.bSave_Preset_2, QtCore.SIGNAL("clicked()"), self.SavePreset_2)
        QtCore.QObject.connect(self.ui.bLoad_Preset_2, QtCore.SIGNAL("clicked()"), self.LoadPreset_2)
        QtCore.QObject.connect(self.ui.bRename_Preset_2, QtCore.SIGNAL("clicked()"), self.RenamePreset_2)
        QtCore.QObject.connect(self.ui.bSave_Preset_3, QtCore.SIGNAL("clicked()"), self.SavePreset_3)
        QtCore.QObject.connect(self.ui.bLoad_Preset_3, QtCore.SIGNAL("clicked()"), self.LoadPreset_3)
        QtCore.QObject.connect(self.ui.bRename_Preset_3, QtCore.SIGNAL("clicked()"), self.RenamePreset_3)
        QtCore.QObject.connect(self.ui.bSave_Preset_4, QtCore.SIGNAL("clicked()"), self.SavePreset_4)
        QtCore.QObject.connect(self.ui.bLoad_Preset_4, QtCore.SIGNAL("clicked()"), self.LoadPreset_4)
        QtCore.QObject.connect(self.ui.bRename_Preset_4, QtCore.SIGNAL("clicked()"), self.RenamePreset_4)
        
#        QtCore.QObject.connect(self.ui.bOK, QtCore.SIGNAL("clicked()"), self.closeEvent)
        QtCore.QObject.connect(self.ui.bAlternativeAlarmPlay, QtCore.SIGNAL("clicked()"), self.AlternativeAlarmPlay)
        QtCore.QObject.connect(self.ui.bcliStartPlay, QtCore.SIGNAL("clicked()"), self.cliStartPlay)
        QtCore.QObject.connect(self.ui.bcliStopPlay, QtCore.SIGNAL("clicked()"), self.cliStopPlay)

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.bAlternativeAlarmPlay.setIcon(QIcon(data.app_path+"img/"+"media-playback-start.png"))
            self.ui.bcliStartPlay.setIcon(QIcon(data.app_path+"img/"+"media-playback-start.png"))
            self.ui.bcliStopPlay.setIcon(QIcon(data.app_path+"img/"+"media-playback-start.png"))
            self.ui.bEditUserName.setIcon(QIcon(data.app_path+"img/"+"edit-rename.png"))
            
            self.ui.bSave_Preset_1.setIcon(QIcon(data.app_path+"img/"+"document-save.png"))
            self.ui.bLoad_Preset_1.setIcon(QIcon(data.app_path+"img/"+"document-open.png"))
            self.ui.bRename_Preset_1.setIcon(QIcon(data.app_path+"img/"+"edit-rename.png"))
            self.ui.bSave_Preset_2.setIcon(QIcon(data.app_path+"img/"+"document-save.png"))
            self.ui.bLoad_Preset_2.setIcon(QIcon(data.app_path+"img/"+"document-open.png"))
            self.ui.bRename_Preset_2.setIcon(QIcon(data.app_path+"img/"+"edit-rename.png"))
            self.ui.bSave_Preset_3.setIcon(QIcon(data.app_path+"img/"+"document-save.png"))
            self.ui.bLoad_Preset_3.setIcon(QIcon(data.app_path+"img/"+"document-open.png"))
            self.ui.bRename_Preset_3.setIcon(QIcon(data.app_path+"img/"+"edit-rename.png"))
            self.ui.bSave_Preset_4.setIcon(QIcon(data.app_path+"img/"+"document-save.png"))
            self.ui.bLoad_Preset_4.setIcon(QIcon(data.app_path+"img/"+"document-open.png"))
            self.ui.bRename_Preset_4.setIcon(QIcon(data.app_path+"img/"+"edit-rename.png"))

        self.setStyleSheet(data.UIStyle) #Add styles

#        self.ui.tabWidget.setTabText(self.ui.tabWidget.indexOf(self.ui.tab_User), _("Name"))
        self.ui.tabWidget.setTabText(self.ui.tabWidget.indexOf(self.ui.tab_Preset), _("Presets"))
        self.ui.tabWidget.setTabText(self.ui.tabWidget.indexOf(self.ui.tab_General), _("General"))
        self.ui.tabWidget.setTabText(self.ui.tabWidget.indexOf(self.ui.tab_Advanced_1), _("Graph") +" 1")
        self.ui.tabWidget.setTabText(self.ui.tabWidget.indexOf(self.ui.tab_Advanced_2), _("Graph") +" 2")
        self.ui.tabWidget.setTabText(self.ui.tabWidget.indexOf(self.ui.tab_Alarm), _("Alarm"))
        self.ui.tabWidget.setTabText(self.ui.tabWidget.indexOf(self.ui.tab_CLI), _("Expert"))

        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Settings")+")")
        self.ui.cOfflineMode.setText(_("Change to Offline Mode\nwhile recording"))
        self.ui.cSilentProfile.setText(_("Change to Silent Profile\nwhile recording"))
        self.ui.cDimmDisplay.setText(_("Dimm display brightness\nat recording start"))
        self.ui.cStopAfterAlarm.setText(_("Stop recording after alarm occured"))
        self.ui.lblUser.setText(_("User name:"))
        self.ui.lblDelayStart.setText(_("Start delay:"))
        self.ui.lblYscale_Movement.setText(_("Y scaling movement (yellow):"))
        self.ui.lblYscale_Sleep_Pattern.setText(_("Y scaling sleep pattern graph (blue):"))
        self.ui.lblYoffset_Sleep_Pattern.setText(_("Y offset sleep pattern graph (blue):"))
        self.ui.lblAlternativeAlarm.setText(_("Alternative alarm:"))
        self.ui.lblAlternativeAlarmNote.setText(_("Type in any program name to be run instead the normal alarm.\nI.e. evilalarm --wakeup."))


        self.ui.lblcliStart.setText(_("This command gets executed before the recording starts:"))
        self.ui.lblcliStop.setText(_("This command gets executed after the recording stops:"))

        LoadPresetNames()

        self.UpdateScreen()

        self.ui.tabWidget.setCurrentIndex(0)  #change to first page





    ###########################################################################
    ## Function:    UpdateScreen
    ## Description: Update window data
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreen(self): ##(Class ConfigWindow)
        if(data.OfflineMode==True): self.ui.cOfflineMode.setChecked(True)
        else: self.ui.cOfflineMode.setChecked(False)
        if(data.SilentProfile==True): self.ui.cSilentProfile.setChecked(True)
        else: self.ui.cSilentProfile.setChecked(False)
        if(data.DimmDisplay==True): self.ui.cDimmDisplay.setChecked(True)
        else: self.ui.cDimmDisplay.setChecked(False)
        if(data.StopAfterAlarm==True): self.ui.cStopAfterAlarm.setChecked(True)
        else: self.ui.cStopAfterAlarm.setChecked(False)
        self.ui.lblUser.setText(_("User name:")+" "+data.user)
        self.ui.lblDelayStart.setText(_("Start delay:")+" "+str(data.DelayStartTime)+" seconds")
        self.ui.SlrDelayStart.setValue(int(data.DelayStartTime))

        self.ui.lblYscale_Movement.setText(_("Y scaling movement (yellow):")+" "+str(data.Yscale_Movement))
        self.ui.SlrYscale_Movement.setValue(int(data.Yscale_Movement))

        self.ui.lblYscale_Sleep_Pattern.setText(_("Y scaling sleep pattern graph (blue):")+" "+str(data.Yscale_Sleep_Pattern))
        self.ui.SlrYscale_Sleep_Pattern.setValue(int(data.Yscale_Sleep_Pattern))

        self.ui.lblYoffset_Sleep_Pattern.setText(_("Y offset sleep pattern graph (blue):")+" "+str(data.Yoffset_Sleep_Pattern))
        self.ui.SlrYoffset_Sleep_Pattern.setValue(int(data.Yoffset_Sleep_Pattern))

        self.ui.cbAlternativeAlarm.removeItem(1)
        if(data.AlternativeAlarm!="evilalarm --wakeup"):
            self.ui.cbAlternativeAlarm.addItem(data.AlternativeAlarm)
            self.ui.cbAlternativeAlarm.setCurrentIndex(1)

        self.ui.txtcliStart.setText(data.cliStart)
        self.ui.txtcliStop.setText(data.cliStop)
        
        self.ui.lblPreset_1.setText(data.preset_names[0])
        self.ui.lblPreset_2.setText(data.preset_names[1])
        self.ui.lblPreset_3.setText(data.preset_names[2])
        self.ui.lblPreset_4.setText(data.preset_names[3])






    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: close window, show start or view window
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event=0):  ##(Class ConfigWindow)
        self.save() #closing the window is equal to saving
        if(data.LastWindow=="ViewWindow"):
            ViewWindow.ShowRecord()
            ViewWindow.show()
        else: StartWindow.show()
        self.close()





    ###########################################################################
    ## Function:    save
    ## Description: Write all data into config
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def save(self): ##(Class ConfigWindow)
        if(self.ui.cOfflineMode.isChecked()==True):
            data.OfflineMode=True
        else:
            data.OfflineMode=False
        config.set('main', 'OfflineMode', str(data.OfflineMode))

        if(self.ui.cSilentProfile.isChecked()==True):
            data.SilentProfile=True
        else:
            data.SilentProfile=False
        config.set('main', 'SilentProfile', str(data.SilentProfile))

        if(self.ui.cDimmDisplay.isChecked()==True):
            data.DimmDisplay=True
        else:
            data.DimmDisplay=False
        config.set('main', 'DimmDisplay', str(data.DimmDisplay))

        if(self.ui.cStopAfterAlarm.isChecked()==True):
            data.StopAfterAlarm=True
        else:
            data.StopAfterAlarm=False
        config.set('main', 'StopAfterAlarm', str(data.StopAfterAlarm))

        data.AlternativeAlarm=str(self.ui.cbAlternativeAlarm.currentText())
        data.cliStart=str(self.ui.txtcliStart.text())
        data.cliStop=str(self.ui.txtcliStop.text())

  #      config.set('main', 'user', str(data.user))
        config.set('main', 'user', data.user.encode( "utf-8" ))
        config.set('main', 'DelayStartTime', str(data.DelayStartTime))
        config.set('main', 'Yscale_Movement', str(data.Yscale_Movement))
        config.set('main', 'Yscale_Sleep_Pattern', str(data.Yscale_Sleep_Pattern))
        config.set('main', 'Yoffset_Sleep_Pattern', str(data.Yoffset_Sleep_Pattern))
        config.set('main', 'AlternativeAlarm', str(data.AlternativeAlarm))

        config.set('main', 'cliStart', str(data.cliStart))
        config.set('main', 'cliStop', str(data.cliStop))



    ###########################################################################
    ## Function:    EditUserName
    ## Description: Open dialog to update user name
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def EditUserName(self): ##(Class ConfigWindow)
        EditUserName()
        self.ui.lblUser.setText(_("User name:")+" "+data.user)




    ###########################################################################
    ## Function:    DelaySliderMoved
    ## Description: Slider event: Update Start delay time
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def DelaySliderMoved(self): ##(Class ConfigWindow)
        x=self.ui.SlrDelayStart.value()
        data.DelayStartTime=x
        self.ui.lblDelayStart.setText(_("Start delay:")+" "+str(data.DelayStartTime)+" "+_("seconds"))




    ###########################################################################
    ## Function:    YscaleSliderMoved_Movement
    ## Description: Slider event: Update Y scaling yellow movement bars
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def YscaleSliderMoved_Movement(self): ##(Class ConfigWindow)
        x=self.ui.SlrYscale_Movement.value()
        data.Yscale_Movement=int(x)
        self.ui.lblYscale_Movement.setText(_("Y scaling movement (yellow):")+" "+str(data.Yscale_Movement))




    ###########################################################################
    ## Function:    YscaleSliderMoved_Sleep_Pattern
    ## Description: Slider event: Update Y scaling blue sleep pattern curve
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def YscaleSliderMoved_Sleep_Pattern(self): ##(Class ConfigWindow)
        x=self.ui.SlrYscale_Sleep_Pattern.value()
        data.Yscale_Sleep_Pattern=int(x)
        self.ui.lblYscale_Sleep_Pattern.setText(_("Y scaling sleep graph (blue):")+" "+str(data.Yscale_Sleep_Pattern))




    ###########################################################################
    ## Function:    YoffsetSliderMoved_Sleep_Pattern
    ## Description: Slider event: Update Y offset blue sleep pattern curve
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def YoffsetSliderMoved_Sleep_Pattern(self): ##(Class ConfigWindow)
        x=self.ui.SlrYoffset_Sleep_Pattern.value()
        data.Yoffset_Sleep_Pattern=int(x)
        self.ui.lblYoffset_Sleep_Pattern.setText(_("Y offset sleep graph (blue):")+" "+str(data.Yoffset_Sleep_Pattern))




    ###########################################################################
    ## Function:    Yscale_Movement_Reset
    ## Description: Button event: Reset Y scaling yellow movement bars
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Yscale_Movement_Reset(self): ##(Class ConfigWindow)
        self.ui.SlrYscale_Movement.setValue(int(data.Yscale_Movement_default))
        self.YscaleSliderMoved_Movement()




    ###########################################################################
    ## Function:    Yscale_Sleep_Pattern_Reset
    ## Description: Button event: Reset Y scaling blue sleep pattern curve
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Yscale_Sleep_Pattern_Reset(self): ##(Class ConfigWindow)
        self.ui.SlrYscale_Sleep_Pattern.setValue(int(data.Yscale_Sleep_Pattern_default))
        self.YscaleSliderMoved_Sleep_Pattern()




    ###########################################################################
    ## Function:    Yoffset_Sleep_Pattern_Reset
    ## Description: Button event: Reset Y offset blue sleep pattern curve
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def Yoffset_Sleep_Pattern_Reset(self): ##(Class ConfigWindow)
        self.ui.SlrYoffset_Sleep_Pattern.setValue(int(data.Yoffset_Sleep_Pattern_default))
        self.YoffsetSliderMoved_Sleep_Pattern()




    ###########################################################################
    ## Function:    DelayStart_Reset
    ## Description: Button event: Reset start delay time
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def DelayStart_Reset(self): ##(Class ConfigWindow)
        self.ui.SlrDelayStart.setValue(int(data.DelayStartTime_default))
        self.DelaySliderMoved()




    ###########################################################################
    ## Function:    AlternativeAlarmPlay
    ## Description: Button event: Run selected script
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def AlternativeAlarmPlay(self):
        cmd=str(self.ui.cbAlternativeAlarm.currentText())
        Notify(_("Run command:")+" "+cmd)
        print "Run command:", cmd
        b = os.popen(cmd+" &")
        b.close()




    ###########################################################################
    ## Function:    cliStartPlay
    ## Description: Button event: Run selected script
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def cliStartPlay(self):
        cmd=str(self.ui.txtcliStart.text())
        Notify(_("Run command:")+" "+cmd)
        print "Run command:", cmd
        b = os.popen(cmd+" &")
        b.close()




    ###########################################################################
    ## Function:    cliStartPlay
    ## Description: Button event: Run selected script
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def cliStopPlay(self):
        cmd=str(self.ui.txtcliStop.text())
        Notify(_("Run command:")+" "+cmd)
        print "Run command:", cmd
        b = os.popen(cmd+" &")
        b.close()





    ###########################################################################
    ## Function:    LoadPreset
    ## Description: Loads the preset names from a file
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def LoadPreset_1(self):
        self.LoadPreset(1)

    def LoadPreset_2(self):
        self.LoadPreset(2)

    def LoadPreset_3(self):
        self.LoadPreset(3)

    def LoadPreset_4(self):
        self.LoadPreset(4)

    def LoadPreset(self, index):
        global config
        f=data.configFolder+data.ConfigFile
        if(os.path.exists(str(f+"_Preset_"+str(index)))):
            print "Load config from preset file", index
            cmd="rm -f "+f+"; cp -f "+f+"_Preset_"+str(index)+" "+f
            print cmd
            b = os.popen(cmd)
            b.close()

            config=LoadConfig()
            ConfigWindow.UpdateScreen()
            Notify(_("Preset loaded"))
            #PrintConfig()
        else:
            QMessageBox.critical(None, _("SleepAnalyser"),_("This preset does not exist yet"))





    ###########################################################################
    ## Function:    WritePreset
    ## Description: Writes the preset names to a file
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def SavePreset_1(self):
        self.SavePreset(1)

    def SavePreset_2(self):
        self.SavePreset(2)

    def SavePreset_3(self):
        self.SavePreset(3)

    def SavePreset_4(self):
        self.SavePreset(4)

    def SavePreset(self, index):
        print "Save config to preset file", index
        self.save()
        WriteConfig()
        f=data.configFolder+data.ConfigFile
        cmd="cp -f "+f+" "+f+"_Preset_"+str(index)
        print cmd
        b = os.popen(cmd)
        b.close()
        Notify(_("Preset saved"))
        #PrintConfig()
        





    ###########################################################################
    ## Function:    RenamePreset
    ## Description: Renames a preset and saves it to a file
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def RenamePreset_1(self):
        self.RenamePreset(1)

    def RenamePreset_2(self):
        self.RenamePreset(2)
        
    def RenamePreset_3(self):
        self.RenamePreset(3)
        
    def RenamePreset_4(self):
        self.RenamePreset(4)

    def RenamePreset(self, index):
        r = QInputDialog.getText(None, _("SleepAnalyser"), _("Preset name:"), 0, data.preset_names[index-1])
        if(r[1]==True): #OK pressed
            data.preset_names[index-1]=unicode(r[0])
            if(data.preset_names[index-1]==""): data.preset_names[index-1]=_("Preset")+" "+str(index)
            self.UpdateScreen()
            
            WritePresetNames()









###########################################################################
###########################################################################
## Function:    Class for date selection window
## Description: All functions needed for date selection window
##                This window is used to select a record of a specific date
##                It is only called from the set view window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmDateWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_DateWindow()
        self.ui.setupUi(self)

        #Signals
        QtCore.QObject.connect(self.ui.cal, QtCore.SIGNAL('selectionChanged()'), self.DateChanged)
        QtCore.QObject.connect(self.ui.bOpen, QtCore.SIGNAL("clicked()"), self.ShowRecord)

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.bOpen.setIcon(QIcon(data.app_path+"img/"+"zoom.png"))

        self.setStyleSheet(data.UIStyle) #Add styles

        self.ui.cal.showToday()

        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Select date")+")")




    ###########################################################################
    ## Function:    DateChanged
    ## Description: Button event: Check how many records are available for selected date
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def DateChanged(self): ##(Class DateWindow)
        date = self.ui.cal.selectedDate()
        date=str(date.toPyDate())
        arr=getFileList(data.RecordFolder,  "*"+date+"*.csv")
        amount=len(arr)

#TODO: visualise dates with records (=> not easily possible, would have to rewrite date dialog)

        txt=string.replace(_("Found % Record(s) for this date."),"%", str(amount))
        self.ui.lblResult.setText(txt)
        if(amount>0):
            self.ui.bOpen.setEnabled(True)
            data.RecordFileSelectedDate=arr[0] #select first file of that date for opening

            for i in range(0, len(data.arrRecordFiles)-1):
                if(data.arrRecordFiles[i]==data.RecordFileSelectedDate):
                    data.arrRecordFilesIndex=i
                    break

        else: self.ui.bOpen.setEnabled(False)





    ###########################################################################
    ## Function:    ShowRecord
    ## Description: Button event: Loads selected record
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ShowRecord(self): ##(Class DateWindow)
        data.RecordFilename=data.RecordFileSelectedDate
        print "Set folder=", data.RecordFolder, ", file=", data.RecordFilename
        DateWindow.hide()
        ViewWindow.ShowRecord()

















###########################################################################
###########################################################################
## Function:    Class for additional info window
## Description: All functions needed for additional info window
##                This window is used to show the additional info in a record
##                It is only called from the view window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmAdditionalInfoWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_AdditionalInfoWindow()
        self.ui.setupUi(self)

        self.setStyleSheet(data.UIStyle) #Add styles

        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Record Info\'s")+")")





    ###########################################################################
    ## Function:    UpdateScreen
    ## Description: Update window data, create text for browser window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreen(self): ##(Class AdditionalInfoWindow)
        shortfolder=string.replace(data.RecordFolder, data.homedir, "~/")
        shortfolder=string.replace(shortfolder, "//", "/") #workaround

        d0=strftime("%A %d. %B %Y  %H:%M", records.StartTime_tuple)
        if(strftime("%d.%m.%Y", records.StartTime_tuple)==strftime("%d.%m.%Y", records.EndTime_tuple)):
            d1=strftime("%H:%M", records.EndTime_tuple) #on same day, only show date once
        else:
            d1=strftime("%a. %d.%m.%Y  %H:%M", records.EndTime_tuple)

        diff_sec=records.Index*records.Sampling_interval*records.Value_smoothing/1000
        diff=time.strftime("%Hh %Mm", time.gmtime(diff_sec))

        if(diff_sec>86400): diff="1 day, "+diff #longer than 24h

        if(records.AlarmIsSet==True): #create text about alarm data
            a0=strftime("%A %d. %B %Y  %H:%M", records.AlarmTimeWindowStart_tuple)
            if(strftime("%d.%m.%Y", records.AlarmTimeWindowStart_tuple)==strftime("%d.%m.%Y", records.AlarmTimeWindowEnd_tuple)):
                a1=strftime("%H:%M", records.AlarmTimeWindowEnd_tuple) #on same day, only show date once
            else:
                a1=strftime("%A %d. %B %Y  %H:%M", records.AlarmTimeWindowEnd_tuple)
            alarm="<b>"+_("Alarm Time Window:")+"</b> "+QtGui.QApplication.translate("", a0, None, QtGui.QApplication.UnicodeUTF8)+" - "+QtGui.QApplication.translate("", a1, None, QtGui.QApplication.UnicodeUTF8) + "<br>"
            if(records.AlarmTime_seconds!=-1): #alarm was set
                if(records.EndTime_seconds>=records.AlarmTime_seconds-60): #alarm got set off during recording (-60 is workaround)
                    alarm=alarm+"<b>"+_("Alarm:")+"</b> "+QtGui.QApplication.translate("", strftime("%A %d. %B %Y  %H:%M", records.AlarmTime_tuple), None, QtGui.QApplication.UnicodeUTF8)+"<br>"
                    if(records.AlarmTimeWindowEnd_seconds<=records.AlarmTime_seconds):
                        alarm=alarm+"<b>"+_("Alarm reason:")+"</b> "+_("End of alarm time window")+"<br>"
                    else:
                        alarm=alarm+"<b>"+_("Alarm reason:")+"</b> "+_("Movements over trigger level")+"<br>"
                else:
                    alarm=alarm+"<b>"+_("Alarm:")+"</b> "+_("Never went off")+"<br>"
            else:
                alarm=alarm+"<b>"+_("Alarm:")+"</b> "+_("Never went off")+"<br>"
            alarm= alarm+"<b>"+_("Alarm trigger level:")+"</b> "+str(records.Trigger)+"<br>"
            if(records.UseAlternativeAlarm==False):
                alarm= alarm+"<b>"+_("Alarm type:")+"</b> "+ _("Normal")
            else:
                alarm= alarm+"<b>"+_("Alarm type:")+"</b> "+_("Alternative alarm")+" \""+str(records.AlternativeAlarm)+"\""
        else:
            alarm="<b>"+_("Alarm:")+"</b> "+_("Not used")

        #create text about general info
        txt= "<b>"+_("File:")+"</b> "+str(shortfolder)+str(data.RecordFilename+"<br>") + \
        "<b>"+_("Date/Time:")+"</b> "+QtGui.QApplication.translate("", d0, None, QtGui.QApplication.UnicodeUTF8)+" - "+d1+"<br>" + \
        "<b>"+_("Duration:")+"</b> "+ diff+"<br>" + \
        "<b>"+_("Note:")+"</b> "+QtGui.QApplication.translate("", records.Note, None, QtGui.QApplication.UnicodeUTF8)+"<br>" + \
        "<b>"+_("User name:")+"</b> "+QtGui.QApplication.translate("", records.User, None, QtGui.QApplication.UnicodeUTF8) +"<br>" + \
        "<b>"+_("Record entries:")+"</b> "+str(records.Index)+" (1/"+str(int(records.Sampling_interval*records.Value_smoothing/1000))+" "+_("seconds")+")<br>" +\
        "<b>"+_("Record version:")+"</b> "+str(records.SWversion) + "<br>"

        txt= txt+"<b>"+_("Condition of record:")+"</b> "+QtGui.QApplication.translate("", records.status, None, QtGui.QApplication.UnicodeUTF8)

        txt=txt+"<hr>"

        if(records.Use_Lucid_Dream_Song==True): #create text about lucid dream option
            if(records.Lucid_Dream_Song_Start_tuple==-1):
                txt= txt+"<b>" + _("Lucid dream song:") +"</b> " + _("Never went off") + "<br>"
            else:
                d=strftime("%A %d. %B %Y  %H:%M", records.Lucid_Dream_Song_Start_tuple)
                txt= txt+"<b>" + _("Lucid dream song start:") +"</b> " + QtGui.QApplication.translate("", d, None, QtGui.QApplication.UnicodeUTF8) + "<br>" + \
                "<b>" + _("Lucid dream song:")+"</b> "+QtGui.QApplication.translate("", records.Lucid_Dream_Song, None, QtGui.QApplication.UnicodeUTF8)  + "<br>" + \
                "<b>" + _("Lucid dream song duration:")+"</b> "+str(int(records.Lucid_Dream_Song_Duration/60)) +" "+ _("minutes")+"<br>" + \
                "<b>" + _("Lucid dream song volume:")+"</b> "+str(records.Lucid_Dream_Song_Volume) + "%"+"<br>"
            txt= txt+"<b>" + _("Lucid dream song delay:")+"</b> "+str(int(records.Lucid_Dream_Song_Delay/60)) +" "+ _("minutes")
        else:
            txt= txt+"<b>" + _("Lucid dream song:") + "</b> " + _("Not used")

        txt=txt+"<hr>"

        if(records.Use_WakeUp_Song==True): #create text about wake up option
            if(records.WakeUp_Song_Start_tuple==-1):
                txt= txt+"<b>" + _("Wake up song:") +"</b> " + _("Never went off")
            else:
                d=strftime("%A %d. %B %Y  %H:%M", records.WakeUp_Song_Start_tuple)
                txt= txt+"<b>" + _("Wake up song start:") +"</b> " + QtGui.QApplication.translate("", d, None, QtGui.QApplication.UnicodeUTF8) + "<br>" + \
                "<b>" + _("Wake up song:")+"</b> "+QtGui.QApplication.translate("", records.WakeUp_Song, None, QtGui.QApplication.UnicodeUTF8) + "<br>" + \
                "<b>" + _("Wake up song duration:")+"</b> "+str(int(records.WakeUp_Song_Duration/60)) +" "+ _("minutes")+"<br>" + \
                "<b>" + _("Wake up song volume:")+"</b> "+str(records.WakeUp_Song_Volume) + "%"
        else:
            txt= txt+"<b>" + _("Wake up song:") + "</b> " + _("Not used")

        txt=txt+"<hr>"

        txt= txt+ alarm

        #create text about statistic data
        txt=txt+"<hr>"
        txt=txt+"<b>"+_("Statistics:")+"</b><table CELLPADDING=2>\n" + \
                "<tr bgcolor=#DCDCDC><td></td>                                                               <td><b>"+_("Total:")+" </b></td>                                     <td><b>"+_("Maximum:")+"</b></td>                                    <td><b>"+_("Count:")+" </b></td>                     <td colspan=2><b>"+_("Time percent:")+"</b></td>\n" + \
                "<tr><td bgcolor=#DCDCDC><b>"+_("Awake")+":</b></td>         <td><nobr>"+strftime("%Hh %Mm ", time.gmtime(records.arrStatData[0][0]))+ "</nobr></td>    <td><nobr>"+strftime(" %Hh %Mm ", time.gmtime(records.arrStatData[0][2]))+"</nobr></td>    <td>"+str(records.arrStatData[0][1])+"</td>    <td>"+str(records.arrStatData[0][3])+"%</td><td><img src=\""+data.app_path+"img/"+"Awake.png"+"\" width="+str(records.arrStatData[0][3]*2)+" height=25</td></tr>\n" + \
                "<tr><td bgcolor=#DCDCDC><b>"+_("Light sleep")+":</b></td>         <td><nobr>"+strftime("%Hh %Mm ", time.gmtime(records.arrStatData[1][0]))+ "</nobr></td>    <td><nobr>"+strftime(" %Hh %Mm ", time.gmtime(records.arrStatData[1][2]))+"</nobr></td>    <td>"+str(records.arrStatData[1][1])+"</td>    <td>"+str(records.arrStatData[1][3])+"%</td><td><img src=\""+data.app_path+"img/"+"Light_Sleep.png"+"\" width="+str(records.arrStatData[1][3]*2)+" height=25</td></tr>\n" + \
                "<tr><td bgcolor=#DCDCDC><b>"+_("Deep sleep")+":</b></td>        <td><nobr>"+strftime("%Hh %Mm ", time.gmtime(records.arrStatData[2][0]))+"</nobr></td>    <td><nobr>"+strftime(" %Hh %Mm ", time.gmtime(records.arrStatData[2][2]))+"</nobr></td>    <td>"+str(records.arrStatData[2][1])+"</td>    <td>"+str(records.arrStatData[2][3])+"%</td><td><img src=\""+data.app_path+"img/"+"Deep_Sleep.png"+"\" width="+str(records.arrStatData[2][3]*2)+" height=25</td></tr>\n" + \
              "</table>\n"

        txt=txt+"<hr>"
        txt=txt+"<b>"+_("Sampling interval:")+"</b> "+str(records.Sampling_interval) + " ms<br>" + \
        "<b>"+_("Sampling smoothing:")+"</b> "+str(records.Sample_smoothing) + "<br>" + \
        "<b>"+_("Value smoothing:")+"</b> "+str(records.Value_smoothing)

        #show text
        self.ui.textBrowser.setHtml("<html><body>"+txt+"</html></body>")





    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: close window, show view window
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event): ##(Class AdditionalInfoWindow)
        ViewWindow.show()




















###########################################################################
###########################################################################
## Function:    Class for alarm setting window
## Description: All functions needed for alarm settings window
##                This window is used to set up the alarm time and window
##                It is only called from the set recording window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmAlarmSettingsWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_AlarmSettingsWindow()
        self.ui.setupUi(self)

        #signals
        QtCore.QObject.connect(self.ui.bSetAlarm, QtCore.SIGNAL("clicked()"), self.SetAlarm)
        QtCore.QObject.connect(self.ui.cbAlternativeAlarm, QtCore.SIGNAL("toggled(bool)"), self.AlternativeAlarm)

        self.setStyleSheet(data.UIStyle) #Add styles

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.bSetAlarm.setIcon(QIcon(data.app_path+"img/"+"SleepAnalyser.png"))

        #texts
        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Set alarm")+")")
        txt=_("The Alarm will go off sometimes during the set time window (as soon as your movement is over the trigger level), but latest at the set alarm time.")
        self.ui.lblNote.setText(txt)
        self.ui.bSetAlarm.setText(_("Set alarm"))
        self.ui.cbAlternativeAlarm.setText(_("Alternative alarm:"))
        self.ui.label_7.setText(_("Alarm Time:"))
        self.ui.label_9.setText(_("Time Window:"))
        self.ui.label_8.setText(_("Hour:"))
        self.ui.label_10.setText(_("Minutes:"))
        self.ui.label_11.setText(_("Minutes:"))

        if(data.UseAlternativeAlarm==True): self.ui.cbAlternativeAlarm.setChecked(True)
        else: self.ui.cbAlternativeAlarm.setChecked(False)




    ###########################################################################
    ## Function:    UpdateScreen
    ## Description: Update window data
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def UpdateScreen(self): ##(Class AlarmSettingsWindow)
        if(data.AlarmHour==-1 or data.AlarmMinute==-1):
            data.AlarmHour=7
            data.AlarmMinute=0
            data.AlarmTimeWindow=30
        if(data.debug==True): print "Alarm time:", data.AlarmHour, ":", data.AlarmMinute
        self.ui.lstAlarmHour.setCurrentRow(int(data.AlarmHour))
        self.ui.lstAlarmMinute.setCurrentRow(int(data.AlarmMinute)/5)
        self.ui.lstAlarmTimeWindow.setCurrentRow(int(data.AlarmTimeWindow)/5-1)




    ###########################################################################
    ## Function:    AlternativeAlarm
    ## Description: Button event: enable/disable alternative alarm
    ## Parameters:  self, check box state
    ## Returns:     none
    ###########################################################################
    def AlternativeAlarm(self, state):
        if(state==True):
            if(data.AlternativeAlarm==""):
                self.ui.cbAlternativeAlarm.setChecked(False)
                QtGui.QMessageBox.warning(self, _("SleepAnalyser"), _("Please first add an alternative alarm command in the settings menu!"), QtGui.QMessageBox.Ok)
            else:
                data.UseAlternativeAlarm=True
        else:
            data.UseAlternativeAlarm=False




    ###########################################################################
    ## Function:    SetAlarm
    ## Description: Button event: set alarm data, close window
    ## Parameters:  self, check box state
    ## Returns:     none
    ###########################################################################
    def SetAlarm(self): ##(Class AlarmSettingsWindow)
        data.UseAlarm=True
        SetRecordingWindow.ui.cbAlarmEnable.setChecked(True)
        data.AlarmHour=int(self.ui.lstAlarmHour.currentItem().text())
        data.AlarmMinute=int(self.ui.lstAlarmMinute.currentItem().text())
        data.AlarmTimeWindow=int(self.ui.lstAlarmTimeWindow.currentItem().text())
#        print data.AlarmHour, data.AlarmMinute, data.AlarmTimeWindow

        SetRecordingWindow.UpdateScreen()
        SetRecordingWindow.show()
        AlarmSettingsWindow.hide()




    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: close window, show recording settings window
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event): ##(Class AlarmSettingsWindow)
        self.SetAlarm()



















###########################################################################
###########################################################################
## Function:    Class for Recording termination window
## Description: All functions needed for the recording termination window
##                This window is used to chose the next action when the user
##              wants to stop the recording
##                It is only called from the recording window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class frmRecordingTerminationWindow(QMainWindow):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the window
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent) # Notice that you must give a parent window as parameter to the constuctor
        try:
            if(data.DeviceIsN900==True): self.setAttribute(Qt.WA_Maemo5StackedWindow) # This attribute makes the whole Stacked Window thing work
        except:
            pass
        self.ui = Ui_RecordingTerminationWindow()
        self.ui.setupUi(self)

        QtCore.QObject.connect(self.ui.bstop_keepRecord, QtCore.SIGNAL("clicked()"), self.KeepRecord)
        QtCore.QObject.connect(self.ui.bstop_removeRecord, QtCore.SIGNAL("clicked()"), self.RemoveRecord)
        QtCore.QObject.connect(self.ui.bcontinue, QtCore.SIGNAL("clicked()"), self.ContinueRecording)

        self.setStyleSheet(data.UIStyle) #Add styles

        if(data.app_path!="./"): #only needed if not started in app path
            self.ui.bstop_keepRecord.setIcon(QIcon(data.app_path+"img/"+"zoom.png"))
            self.ui.bstop_removeRecord.setIcon(QIcon(data.app_path+"img/"+"edittrash.png"))
            self.ui.bcontinue.setIcon(QIcon(data.app_path+"img/"+"krec.png"))

        self.setWindowTitle(_("SleepAnalyser")+" ("+_("Recording")+")")
        self.ui.bcontinue.setText(_("Continue recording"))
        self.ui.bstop_keepRecord.setText(_("Stop recording and keep the record"))
        self.ui.bstop_removeRecord.setText(_("Stop recording and delete the record"))





    ###########################################################################
    ## Function:    KeepRecord
    ## Description: button event:Stop measurement, close recording window, open view window with latest record
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def KeepRecord(self): ##(Class RecordingTerminationWindow)
        print "Stop recording and keep record"
        StopMeasurement()
        print "*", strftime("%x %X"), "KeepRecord => DeleteAlarm" #for log file
        DeleteAlarm() #delete Alarm if existing
        records.AlarmIsSet=False
        data.LastRecord=data.RecordFilename #set to corrent file for viewwindow
        config.set('main', 'LastRecord', str(data.LastRecord))
        print "Show record in ViewWindow:", data.LastRecord
        RecordWindow.hide()
        self.hide()
        StartWindow.ShowViewWindow()
        app.processEvents() #force ui update
        SwitchModes(False) #change back to normal mode and general profile if requested





    ###########################################################################
    ## Function:    RemoveRecord
    ## Description: button event: Stop measurement, delete record
    ##                close recording window, open start window
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def RemoveRecord(self): ##(Class RecordingTerminationWindow)
        print "Stop recording and remove record"
        StopMeasurement()
        print "*", strftime("%x %X"), "RemoveRecord => DeleteAlarm" #for log file
        DeleteAlarm() #delete Alarm if existing
        records.AlarmIsSet=False
        print "Delete RecordFile: "+data.RecordFilename
        try:
            os.remove(data.RecordFolder+data.RecordFilename)
        except:
            print "Can not delete it. It might not exist"
        RecordWindow.hide()
        self.hide()
        StartWindow.show()
        app.processEvents() #force ui update
        SwitchModes(False) #change back to normal mode and general profile if requested



    ###########################################################################
    ## Function:    ContinueRecording
    ## Description: button event: close window, continue recording
    ## Parameters:  self
    ## Returns:     none
    ###########################################################################
    def ContinueRecording(self): ##(Class RecordingTerminationWindow)
        print "continue recording"
        self.hide()
        RecordWindow.show()




    ###########################################################################
    ## Function:    closeEvent
    ## Description: Close event: IGNORE CALL, DO NOT CLOSE WINDOW
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def closeEvent(self, event): ##(Class RecordingTerminationWindow)
        event.ignore() #do not close
        self.KeepRecord() #interprete closing as "stop and keep file"


















###########################################################################
###########################################################################
## Function:    Class for Display status deamon
## Description: All functions needed for the dbus monitoring deamon
## Parameters:  none
## Returns:     none
###########################################################################
###########################################################################
class DisplayStatusDaemon():
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the DBUS monitoring deamon
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self):
        if(data.DeviceIsN900==True): #We are on a N900
            loop = DBusQtMainLoop(set_as_default=True)
            system_bus = dbus.SystemBus()
            system_bus.add_signal_receiver(self.DisplayStatus,
                path='/com/nokia/mce/signal',
                signal_name='display_status_ind',
                dbus_interface='com.nokia.mce.signal')

            if(data.debug==True): print "DisplayStatusDaemon started"




    ###########################################################################
    ## Function:    DisplayStatus
    ## Description: DBUS signal event: update display status data
    ## Parameters:  self, parameter pointer
    ## Returns:     none
    ###########################################################################
    def DisplayStatus(self, *params):
        global RecordWindow
        tmp=str(params)
        tmp=string.split(tmp, "'")
        if(tmp[1]=="off"): data.DisplayIsOn=False
        elif(tmp[1]=="on"):
            data.DisplayIsOn=True
            if(RecordWindow.isHidden()==False):
                if(data.debug==True): print "Update RecordWindow screen & graph"

                GenerateStatisticData()
                Generate_and_show_GraphBitmap(RecordWindow)
                RecordWindow.Canvas.update()
                RecordWindow.UpdateScreen()



















###########################################################################
###########################################################################
## Function:    Class for graph canvas
## Description: All functions needed for graph canvas
##                This canvas shows the graph. It is used in the recoding and view window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class GraphCanvas(QtGui.QWidget):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the canvas
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent):
        QMainWindow.__init__(self, parent)
        self.setGeometry(0, 0, 770, data.GraphBitMapHeight)




    ###########################################################################
    ## Function:    paintEvent
    ## Description: load bitmap into canvas
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def paintEvent(self, event):
        if(data.DisplayIsOn==False and data.run==True):
            if(data.debug==True): print "-> o Do not paint event (Graph), Display is off!"
            return
        else:
            if(data.debug==True): print "-> x paint event (Graph)"

        paint = QtGui.QPainter()
        paint.begin(self)
        paint.drawPixmap(0, 0, data.GraphBitMap) #load graph from Bitmap

        paint.end()

        #Scroll graph to end, as there is the most interesting part
        if(data.ScrollToEnd==True):
            if(data.run==True): RecordWindow.ui.ScrollArea.ensureVisible(data.GraphBitMapLen+100, 0, 0 , 0)
            else:
                ViewWindow.ui.ScrollArea.ensureVisible(data.GraphBitMapLen+100, 0, 0 , 0)
            data.ScrollToEnd=False















###########################################################################
###########################################################################
## Function:    Class for statistic canvas
## Description: All functions needed for statistic canvas
##                This canvas shows the statistics in the view window
## Parameters:  QMainWindow
## Returns:     none
###########################################################################
###########################################################################
class StatCanvas(QtGui.QWidget):
    ###########################################################################
    ## Function:    __init__
    ## Description: Initialises the canvas
    ## Parameters:  self, parent
    ## Returns:     none
    ###########################################################################
    def __init__(self, parent):
        QMainWindow.__init__(self, parent)
        self.setGeometry(406, 280, data.StatisticsBitMapLen, data.StatisticsBitMapHeight)




    ###########################################################################
    ## Function:    paintEvent
    ## Description: load bitmap into canvas
    ## Parameters:  self, event
    ## Returns:     none
    ###########################################################################
    def paintEvent(self, event):
        if(data.DisplayIsOn==False and data.run==True):
            if(data.debug==True): print "-> o Do not paint event (Statistics), Display is off!"
            return
        else:
            if(data.debug==True): print "-> x paint event (Statistics)"

        paint = QtGui.QPainter()
        paint.begin(self)
        paint.setBackgroundMode(QtCore.Qt.TransparentMode)
        paint.drawPixmap(0, 0, data.StatisticsBitMap) #load statistics from Bitmap
        paint.end()










###########################################################################
## Function:    Generate_and_show_GraphBitmap
## Description: Initialise, generate and show graph bitmap
## Parameters:  canvas to be drawn on
## Returns:     none
###########################################################################
def Generate_and_show_GraphBitmap(self):
    if(data.debug==True): print "Generate Graph Bitmap"

    Xscale=data.arrXscale[data.Xscale]
    data.GraphBitMapLen=max((records.LastIndex+5)/Xscale + 2 , 649) #628 minimum graph width
    data.GraphBitMapHeight=data.Ysize+34

    try:
        data.GraphBitMap = QtGui.QPixmap(data.GraphBitMapLen, data.GraphBitMapHeight) #create bitmap for graph
        DrawGraph(data.GraphBitMap, QtGui) # draw graph
    except:
        print "Error while creating the graph bitmap"

    self.Canvas.setGeometry(0, 0, data.GraphBitMapLen, data.GraphBitMapHeight)  #set size of canvas to graph

    self.ui.CanvasWidget.setMinimumSize(data.GraphBitMapLen, data.GraphBitMapHeight)  #set size of canvas widget to graph

    data.ScrollToEnd=True

    self.ui.CanvasWidget.hide() #needed? Might force a reload
    self.ui.CanvasWidget.show()
    self.UpdateScreen()





###########################################################################
## Function:    GenerateStatisticsBitmap
## Description: Initialise and generate statistics bitmap
## Parameters:  none
## Returns:     none
###########################################################################
def GenerateStatisticsBitmap():
    if(data.debug==True): print "Generate Statistics Bitmap"

    try:
        data.StatisticsBitMap = QtGui.QPixmap(data.StatisticsBitMapLen, data.StatisticsBitMapHeight) #create bitmap for graph
        DrawStatistic(data.StatisticsBitMap, QtGui, QtCore) #draw statistics
    except:
        print "Error while creating the statistics bitmap"







###########################################################################
main() # great, now we can start it.
